require 'etc'
require 'fileutils'

###############################################################################
# 環境変数のチェック
###############################################################################
if ENV['PAGER'] == nil || ENV['PAGER'] == ""
    if File.exist?("/usr/bin/lv") || File.exist?("/usr/local/bin/lv")
        ENV['PAGER'] = "lv"
    elsif File.exist?("/usr/bin/less") || File.exist?("/usr/local/bin/less") 
        ENV['PAGER'] = "less"
    else
        ENV['PAGER'] = "more"
    end
end

if ENV['EDITOR'] == nil || ENV['EDITOR'] == ""
    if File.exist?("/usr/bin/vim") || File.exist?("/usr/local/bin/vim") 
        ENV['EDITOR'] = "vim"
    else
        ENV['EDITOR'] = "vi"
    end
end

if ENV['SHELL'] == nil || ENV['SHELL'] == "" 
    if File.exist?("/bin/zsh") || File.exist?("/usr/bin/zsh") || File.exist?("/usr/local/bin/zsh") 
        ENV['SHELL'] = "zsh"
    elsif File.exist?("/bin/bash") || File.exist?("/usr/bin/bash") || File.exist?("/usr/local/bin/bash") 
        ENV['SHELL'] = "bash"
    elsif File.exist?("/bin/tcsh") || File.exist?("/usr/bin/tcsh") || File.exist?("/usr/local/bin/tcsh") 
        ENV['SHELL'] = "tcsh"
     else
        ENV['SHELL'] = "sh"
    end
end

###############################################################################
# 日本語の設定
###############################################################################

# KCODE
if `uname -s`.index("CYGWIN") || ENV['LANG'] == "ja_JP.SJIS" || ENV['LANG'] == "ja_JP.sjis"
    $KCODE = "SJIS"
elsif ENV['LANG'] != nil && (ENV['LANG'].index("utf") || ENV['LANG'].index("UTF")) || `uname -s`.index("Darwin")
    $KCODE = "UTF8"
else
    $KCODE = "EUC"
end

# iconv settings
set_iconv_kanji_code_name "sjis", "sjis"
set_iconv_kanji_code_name "eucjp", "eucjp"
set_iconv_kanji_code_name "utf8", "utf-8"
set_iconv_kanji_code_name "utf8-mac", "utf-8-mac"

###############################################################################
# キーマップ
###############################################################################
keymap   KEY_UP,        27,   91,   65,   -1,      -1,     -1, -1,-1,-1,-1
keymap   KEY_UP,        27,   79,   65,   -1,       -1,    -1, -1,-1,-1,-1
keymap   KEY_DOWN,      27,   91,   66,   -1,      -1,     -1, -1,-1,-1,-1
keymap   KEY_DOWN,      27,   79,   66,   -1,       -1,    -1, -1,-1,-1,-1
keymap   KEY_RIGHT,     27,   91,   67,   -1,      -1,     -1, -1,-1,-1,-1
keymap   KEY_RIGHT,     27,   79,   67,   -1,       -1,    -1, -1,-1,-1,-1
keymap   KEY_LEFT,      27,   91,   68,   -1,      -1,     -1, -1,-1,-1,-1
keymap   KEY_LEFT,      27,   79,   68,   -1,       -1,    -1, -1,-1,-1,-1
keymap   KEY_INSERT,    27,   91,   50,   126,     -1,     -1, -1,-1,-1,-1
keymap   KEY_DELETE,    27,   91,   51,   126,     -1,     -1, -1,-1,-1,-1
keymap   KEY_BACKSPACE, 127,  -1,   -1,   -1,      -1,     -1, -1,-1,-1,-1
keymap   KEY_HOME,      27,   79,   72,   -1,      -1,     -1, -1,-1,-1,-1
keymap   KEY_HOME,      27,   91,   72,   -1,      -1,     -1, -1,-1,-1,-1
keymap   KEY_HOME,      27,   91,   55,   126,     -1,     -1, -1,-1,-1,-1
keymap   KEY_HOME,      27,   91,   0,    -1,      -1,     -1, -1,-1,-1,-1
keymap   KEY_HOME,      27,   91,   49,   126,      -1,    -1, -1,-1,-1,-1
keymap   KEY_END,       27,   79,   70,   -1,      -1,     -1, -1,-1,-1,-1
keymap   KEY_END,       27,   91,   70,   -1,      -1,     -1, -1,-1,-1,-1
keymap   KEY_END,       27,   91,   56,   126,     -1,     -1, -1,-1,-1,-1
keymap   KEY_END,       27,   91,   101,  -1,      -1,     -1, -1,-1,-1,-1
keymap   KEY_END,       27,   91,   52,   126,     -1,     -1, -1,-1,-1,-1
keymap   KEY_PAGEUP,    27,   91,   53,   126,     -1,     -1, -1,-1,-1,-1
keymap   KEY_PAGEUP,    27,   91,   73,   -1,      -1,     -1, -1,-1,-1,-1
keymap   KEY_PAGEDOWN,  27,   91,   54,   126,     -1,     -1, -1,-1,-1,-1
keymap   KEY_PAGEDOWN,  27,   91,   71,   -1,      -1,     -1, -1,-1,-1,-1
keymap   KEY_F1,        27,   91,   49,   49,      126,    -1, -1,-1,-1,-1
keymap   KEY_F1,        27,   79,   80,   -1,      -1,     -1, -1,-1,-1,-1
keymap   KEY_F1,        27,   91,   91,   65,      -1,     -1, -1,-1,-1,-1
keymap   KEY_F2,        27,   91,   49,   50,      126,    -1, -1,-1,-1,-1
keymap   KEY_F2,        27,   79,   81,   -1,      -1,     -1, -1,-1,-1,-1
keymap   KEY_F2,        27,   91,   91,   66,      -1,     -1, -1,-1,-1,-1
keymap   KEY_F3,        27,   91,   49,   51,      126,    -1, -1,-1,-1,-1
keymap   KEY_F3,        27,   79,   82,   -1,      -1,     -1, -1,-1,-1,-1
keymap   KEY_F3,        27,   91,   91,   67,      -1,     -1, -1,-1,-1,-1
keymap   KEY_F4,        27,   91,   49,   52,      126,    -1, -1,-1,-1,-1
keymap   KEY_F4,        27,   79,   83,   -1,      -1,     -1, -1,-1,-1,-1
keymap   KEY_F4,        27,   91,   91,   68,      -1,     -1, -1,-1,-1,-1
keymap   KEY_F5,        27,   91,   49,   53,      126,    -1, -1,-1,-1,-1
keymap   KEY_F5,        27,   91,   91,   69,      -1,     -1, -1,-1,-1,-1
keymap   KEY_F6,        27,   91,   49,   55,      126,    -1, -1,-1,-1,-1
keymap   KEY_F7,        27,   91,   49,   56,      126,    -1, -1,-1,-1,-1
keymap   KEY_F8,        27,   91,   49,   57,      126,    -1, -1,-1,-1,-1
keymap   KEY_F9,        27,   91,   50,   48,      126,    -1, -1,-1,-1,-1
keymap   KEY_F10,       27,   91,   50,   49,      126,    -1, -1,-1,-1,-1
keymap   KEY_F11,       27,   91,   50,   51,      126,    -1, -1,-1,-1,-1
keymap   KEY_F12,       27,   91,   50,   52,      126,    -1, -1,-1,-1,-1

keymap   KEY_META_LEFT, 27,   91,   51,    68,      -1,    -1, -1,-1,-1,-1 # gnome-terminal
keymap   KEY_META_LEFT, 27,   91,   49,    59,      51,    68, -1,-1,-1,-1 # xterm
keymap   KEY_META_LEFT, 27,   91,   49,    59,      57,    68, -1,-1,-1,-1 # mlterm
keymap   KEY_META_LEFT, 27,   79,   49,    59,      51,    68, -1,-1,-1,-1 # screen
#keymap   KEY_META_LEFT, 27,   79,   68,    -1,      -1,    -1, -1,-1,-1,-1 # iTerm

keymap   KEY_META_RIGHT,27,   91,   51,    67,      -1,    -1, -1,-1,-1,-1 # gnome-terminal
keymap   KEY_META_RIGHT,27,   91,   49,    59,      51,    67, -1,-1,-1,-1 # xterm
keymap   KEY_META_RIGHT,27,   91,   49,    59,      57,    67, -1,-1,-1,-1 # mlterm
keymap   KEY_META_RIGHT,27,   79,   49,    59,      51,    67, -1,-1,-1,-1 # screen
#keymap   KEY_META_RIGHT,27,   79,   67,    -1,      -1,    -1, -1,-1,-1,-1 # iTerm

keymap   KEY_META_UP,   27,   91,   51,    65,      -1,    -1, -1,-1,-1,-1 # gnome-terminal
keymap   KEY_META_UP,   27,   91,   49,    59,      51,    65, -1,-1,-1,-1 # xterm
keymap   KEY_META_UP,   27,   91,   49,    59,      57,    65, -1,-1,-1,-1 # mlterm
keymap   KEY_META_UP,   27,   79,   49,    59,      51,    65, -1,-1,-1,-1 # screen
#keymap   KEY_META_UP,   27,   79,   65,    -1,      -1,    -1, -1,-1,-1,-1 # iTerm

keymap   KEY_META_DOWN, 27,   91,   51,    66,      -1,    -1, -1,-1,-1,-1 # gnome-terminal
keymap   KEY_META_DOWN, 27,   91,   49,    59,      51,    66, -1,-1,-1,-1 # xterm
keymap   KEY_META_DOWN, 27,   91,   49,    59,      57,    66, -1,-1,-1,-1 # mlterm
keymap   KEY_META_DOWN, 27,   79,   49,    59,      51,    66, -1,-1,-1,-1 # screen
#keymap   KEY_META_DOWN, 27,   79,   66,    -1,      -1,    -1, -1,-1,-1,-1 # iTerm

###############################################################################
# フック
###############################################################################
def atexit_hook_default()
    save_filer_stat()
    0
end

def mloop_hook_default()
    0
end

def fdclone_type_cursor_move_hook()
    vo = view_filer()
    maxy = cursor_maxy()
    
    if vo == "all" || vo == "all2" || vo == "one_dir" then
         if cursor() >= scrolltop() + maxy then
             set_scrolltop( (cursor()/maxy)*maxy )
         end
         if cursor() < scrolltop() then
             set_scrolltop( (cursor()/maxy)*maxy )
         end
    elsif vo == "one_dir2" then
        if cursor() >= scrolltop() + maxy*2 then
             set_scrolltop( (cursor()/(maxy*2))*(maxy*2) )
        end
        if cursor() < scrolltop() then
            set_scrolltop( (cursor()/(maxy*2))*(maxy*2) )
        end
    elsif vo == "one_dir3" then
        if cursor() >= scrolltop() + maxy*3 then
             set_scrolltop( (cursor()/(maxy*3))*(maxy*3) )
        end
        if cursor() < scrolltop() then
            set_scrolltop( (cursor()/(maxy*3))*(maxy*3) )
        end
    elsif vo == "one_dir5" then
        if cursor() >= scrolltop() + maxy*5 then
             set_scrolltop( (cursor()/(maxy*5))*(maxy*5) )
        end
        if cursor() < scrolltop() then
            set_scrolltop( (cursor()/(maxy*5))*(maxy*5) )
        end
    end
end

def afu_type_cursor_move_hook()
    vo = view_filer()
    maxy = cursor_maxy()
    
    if vo == "all" || vo == "all2" || vo == "one_dir" then
         if cursor() >= scrolltop() + maxy-2 then
             set_scrolltop(cursor() -maxy+2)
         end
         if cursor() < scrolltop() then
             set_scrolltop(cursor())
         end
    elsif vo == "one_dir2" then
        if cursor() >= scrolltop() + maxy*2 then
             set_scrolltop( (cursor()/(maxy*2))*(maxy*2) )
        end
        if cursor() < scrolltop() then
            set_scrolltop( (cursor()/(maxy*2))*(maxy*2) )
        end
    elsif vo == "one_dir3" then
        if cursor() >= scrolltop() + maxy*3 then
             set_scrolltop( (cursor()/(maxy*3))*(maxy*3) )
        end
        if cursor() < scrolltop() then
            set_scrolltop( (cursor()/(maxy*3))*(maxy*3) )
        end
    elsif vo == "one_dir5" then
        if cursor() >= scrolltop() + maxy*5 then
             set_scrolltop( (cursor()/(maxy*5))*(maxy*5) )
        end
        if cursor() < scrolltop() then
            set_scrolltop( (cursor()/(maxy*5))*(maxy*5) )
        end
    end
end

$option_scroll_type = "fdclone"

def cursor_move_hook(i)
    if $option_scroll_type == "afu"
        afu_type_cursor_move_hook()
    elsif $option_scroll_type == "original"
        fdclone_type_cursor_move_hook()
    elsif $option_scroll_type == "fdclone"
        fdclone_type_cursor_move_hook()
    else
        fdclone_type_cursor_move_hook()
    end
end

###############################################################################
# rubyによって定義される mfiler2 API 
###############################################################################
def is_adir_right()
    return !is_adir_left()
end

def adir_num()
    if is_adir_left()
        return 0 
    else 
        return 1
    end
end

def sdir_num()
    if is_adir_left()
        return 1
    else 
        return 0
    end
end

def toggle_xterm_next()
    xterm_next_toggle()
end

def toggle_xterm()
    xterm_toggle()
end

def excute_cursor_file()
    if `uname -s`.index("CYGWIN")
        if cursor_ext() == "pl"
            cmdline2('perl ./%f ', 0, nil)
        elsif cursor_ext() == "py"
            cmdline2('python ./%f ', 0, nil)
        elsif cursor_ext() == "rb"
            cmdline2('ruby ./%f ', 0, nil)
        elsif cursor_ext() == "exe"
            cmdline2('./%f ', 0, nil)
        else
            cmdline2(' ./%f', 1, nil)
        end
    else
        if File.executable?(adir_path() + cursor_name())
            if File.directory?(adir_path() + cursor_name())
                cmdline2(' ./%f', 1, nil)
            else
                cmdline2('./%f ', 0, nil) 
            end
        elsif cursor_ext() == "pl"
            cmdline2('perl ./%f ', 0, nil)
        elsif cursor_ext() == "py"
            cmdline2('python ./%f ', 0, nil)
        elsif cursor_ext() == "rb"
            cmdline2('ruby ./%f ', 0, nil)
        else
            cmdline2(' ./%f', 1, nil)
        end
    end
end

def menu_attr()
    if $sftp_path && is_adir_right()
    else
        menu('attribute')
    end
end

def menu_grep()
    if $sftp_path && is_adir_right()
    else
        menu('grep_menu')
    end
end

def menu_mark()
    if $sftp_path && is_adir_right
    else
        menu('mark_menu')
    end
end

def menu_extra()
    menu('extra-menu')
end

def menu_sudo()
    menu('sudo-menu')
end

def menu_find()
    menu('find')
end

def menu_copy()
    menu('copy_menu')
end

def menu_move()
    menu('move_menu')
end

def menu_delete()
    menu('delete_menu')
end

def menu_sort()
    menu('sort')
end

def menu_all()
    menu('all')
end

def menu_my_commands()
    menu('my_commands')
end

def move_back()
    dir_back()
end

def mfiler2_help()
    help()
end

def tab_menu_select()
    tab_select()
end

def tab_open(num)
    open_tab(num)
end

def job_select()
    select_job()
end

def job_forground(num)
    forground_job(num)
end

def start_isearch()
    isearch()
end

def start_isearch_partmatch()
    isearch_partmatch()
end

def isearch_partmatch_toggle()
    isearch_partmatch2(!is_isearch_partmatch2)
end

def jump_by_cmdline1()
    cmdline2("%r%Q dir_move %D1", -1, nil)
end

def jump_by_cmdline2()
    cmdline2("%r%Q dir_move ''", -1, nil)
end

defmenu("Jump",
    "cmdline (j)ump(default current directory)", KEY_j, "jump_by_cmdline1()",
    "cmdline (J)ump(root directory)", KEY_J, "jump_by_cmdline2()",
    "(h)istory", KEY_h, "menu_dir_history()"
)

def menu_jump()
    menu('Jump')
end

defmenu("sHell",
    "send cmdline to (m)filer2 shell", KEY_m, "cmdline2('%s ', 0, nil)",
    "send cmdline to extern (s)hell($SHELL)", KEY_s, "cmdline2('%S ', 0, nil)",
    "send cmdline to (r)uby", KEY_r, "cmdline2('%r ', 0, nil)",
    "send cmdline to other (x)terminal with extern shell($SHELL)", KEY_x, "cmdline2('%t ', 0, nil)", 
    "cmdline completion $PATH re(h)ash",           KEY_h, "rehash()"
)

def menu_shell()
    menu('sHell')
end

def run_file_manager()
    if `uname -s`.index("CYGWIN")
        shell('%Q cygstart $(cygpath -w -a .) &', 'explore')
    elsif `uname -s`.index("Darwin")
        shell('%Q open . &', 'Finder')
    else
        gnome = false
        kde = false
        xfce = false
        `ps ax`.each {|line|
            prog_name = line.split[4]
            if prog_name == "nautilus"
                gnome = true
            elsif prog_name == "kdeinit:"
                kde = true
            elsif prog_name == "xfdesktop"
                xfce = true
            end
        }
        if gnome
            shell('%Q nautilus file://"$PWD" 2>/dev/null &', 'nautilus')
        elsif kde
            shell('%Q konqueror "$PWD" 2>/dev/null &', 'konqueror')
        elsif xfce
            shell('%Q thunar "$PWD" 2>/dev/null &', 'thunar')
        end
    end
end

def _select_menu(title, select_str, flg_mfiler_view)
    if !mis_curses
        puts "curses is not running..."
        return
    end

    $scroll_top = 0
    $cursor = 0
    $select_str = select_str
    $title = title

    def view_hook()
        height = 0
        if $select_str.size > mgetmaxy - 2
            height = mgetmaxy - 2
        else
            height = $select_str.size+2
        end

        mbox(0,2, mgetmaxx-5, height)
        mmvprintw(0, 4, $title)
        n = $scroll_top
        while n < $select_str.size
            if $cursor == n
                mattron(MA_REVERSE)
            end

            if n - $scroll_top+2 < mgetmaxy - 2
                mmvprintw(n - $scroll_top + 1, 3, "%s", $select_str[n])
            end

            if $cursor == n
                mattroff()
            end
            
            n = n + 1
        end
    end

    while true
        ### view ###
        mclear
        if flg_mfiler_view 
            mfiler_view()
        end
        view_hook()
        mrefresh()

        ### input ###
        key = mgetch()

        if key[1] == 14 || key[1] == KEY_DOWN
            $cursor = $cursor + 1
        end
        if key[1] == 16 || key[1] == KEY_UP
            $cursor = $cursor -1
        end
        if key[1] == 4 || key[1] == KEY_PAGEDOWN
            $cursor = $cursor + mgetmaxy / 2
        end
        if key[1] == 21 || key[1] == KEY_PAGEUP
            $cursor = $cursor - mgetmaxy / 2
        end
        if key[1] == 12
            mclear
            mrefresh
        end

        if key[1] == ?q || key[1] == 3 || key[1] == 27 || key[1] == 7
            $cursor = -1
            break
        end
        
        if key[1] == 10 || key[1] == 13
            break
        end

        ### modification ###
        if $cursor < 0
            $cursor = 0
        end
        if $cursor >= $select_str.size
            $cursor = $select_str.size - 1
        end
        if $cursor < $scroll_top
             $scroll_top = $cursor
        end
        if $cursor > $scroll_top + mgetmaxy() - 5
            $scroll_top = $scroll_top + ($cursor-$scroll_top-mgetmaxy()) + 5
        end
    end

    def view_hook()
    end

    return $cursor
end

def select_menu(title, select_str)
    _select_menu(title, select_str, false)
end

def select_menu_mfiler(title, select_str)
    _select_menu(title, select_str, true)
end

def msystem(cmd)
    curses_on_off = mis_curses()

    if curses_on_off
        mendwin()
    end

    set_signal_clear()

    result = `#{cmd}`

    set_signal_mfiler2()

    if curses_on_off
        minitscr()
    end

    return result
end

def is_one_dir()
    vo = view_filer();

    return vo == "one_dir" || vo == "one_dir2" || vo == "one_dir3" || vo == "one_dir5";
end

def glob_mark(glob)
    garray = Dir[glob]

    i = 0
    while i < cursor_max()
        fname = file_name(i)
        
        if !File.directory?(adir_path() + fname)
            garray.each {|item|
                if item == fname
                    mark_on(i)
                    break
                end
            }
        end

        i = i + 1
    end
end

def glob_mark_dir(glob)
    garray = Dir[glob]

    i = 0
    while i < cursor_max()
        fname = file_name(i)
        
        if File.directory?(adir_path() + fname) && fname != ".." && fname != "."
            garray.each {|item|
                if item == fname
                    mark_on(i)
                    break
                end
            }
        end

        i = i + 1
    end
end

def glob_mark_all(glob)
    garray = Dir[glob]

    i = 0
    while i < cursor_max()
        fname = file_name(i)
        
        if fname != ".." && fname != "."
            garray.each {|item|
                if item == fname
                    mark_on(i)
                    break
                end
            }
        end

        i = i + 1
    end
end

def regex_mark(regex)
    i = 0
    while i < cursor_max()
        fname = file_name(i)
        
        if !File.directory?(adir_path() + fname)
            if fname =~ regex
                mark_on(i)
            end
        end
        
        i = i + 1
    end
end

def regex_mark_dir(regex)
    i = 0
    while i < cursor_max()
        fname = file_name(i)
        
        if File.directory?(adir_path() + fname) && fname != ".." && fname != "."
            if fname =~ regex
                mark_on(i)
            end
        end
        
        i = i + 1
    end
end

def regex_mark_all(regex)
    i = 0
    while i < cursor_max()
        fname = file_name(i)
        
        if fname != ".." && fname != "."
            if fname =~ regex
                mark_on(i)
            end
        end
        
        i = i + 1
    end
end

def mark_directory()
    i = 0
    while i < cursor_max()
        fname = file_name(i)
        
        if File.directory?(adir_path() + fname) && fname != ".." && fname != "."
            mark_on(i)
        end
        
        i = i + 1
    end
end

def mark_executable()
    i = 0
    while i < cursor_max()
        fname = file_name(i)
        
        if File.executable?(adir_path() + fname) && !File.directory?(adir_path() + fname) && fname != ".." && fname != "."
            mark_on(i)
        end
        
        i = i + 1
    end
end

def mark_symlink()
    i = 0
    while i < cursor_max()
        fname = file_name(i)
        
        if File.symlink?(adir_path() + fname) && fname != ".." && fname != "."
            mark_on(i)
        end
        
        i = i + 1
    end
end

def mark_user(user)
    begin
        pwd = Etc.getpwnam(user)

        i = 0
        while i < cursor_max()
            fname = file_name(i)

            if !File.directory?(adir_path() + fname) && File.stat(adir_path() + fname).uid == pwd.uid && fname != ".." && fname != "."
                mark_on(i)
            end
            
            i = i + 1
        end
    rescue
        message("invalid user name")
    end
end

def mark_user_dir(user)
    begin
        pwd = Etc.getpwnam(user)

        i = 0
        while i < cursor_max()
            fname = file_name(i)

            if File.directory?(adir_path() + fname) && File.stat(adir_path() + fname).uid == pwd.uid && fname != ".." && fname != "."
                mark_on(i)
            end
            
            i = i + 1
        end
    rescue
        message("invalid user name")
    end
end

def mark_user_all(user)
    begin
        pwd = Etc.getpwnam(user)

        i = 0
        while i < cursor_max()
            fname = file_name(i)

            if File.stat(adir_path() + fname).uid == pwd.uid && fname != ".." && fname != "."
                mark_on(i)
            end
            
            i = i + 1
        end
    rescue
        message("invalid user name")
    end
end

def mark_group(group)
    begin
        grp = Etc.getgrnam(group)

        i = 0
        while i < cursor_max()
            fname = file_name(i)

            if !File.directory?(adir_path() + fname) && File.stat(adir_path() + fname).gid == grp.gid && fname != ".." && fname != "."
                mark_on(i)
            end
            
            i = i + 1
        end
    rescue
        message("invalid group name")
    end
end

def mark_group_dir(group)
    begin
        grp = Etc.getgrnam(group)

        i = 0
        while i < cursor_max()
            fname = file_name(i)

            if File.directory?(adir_path() + fname) && File.stat(adir_path() + fname).gid == grp.gid && fname != ".." && fname != "."
                mark_on(i)
            end
            
            i = i + 1
        end
    rescue
        message("invalid group name")
    end
end

def mark_group_all(group)
    begin
        grp = Etc.getgrnam(group)

        i = 0
        while i < cursor_max()
            fname = file_name(i)

            if File.stat(adir_path() + fname).gid == grp.gid && fname != ".." && fname != "."
                mark_on(i)
            end
            
            i = i + 1
        end
    rescue
        message("invalid group name")
    end
end

=begin
def mark_clear()
    i = 0
    while i < cursor_max()
        mark_off(i)
        i = i + 1
    end
end
=end

def mark_range_begin()
    $mark_range_begin = cursor()
end

def mark_range_end()
    i = $mark_range_begin
    while i <= cursor() && i < cursor_max()
        mark_on(i)
        i = i + 1
    end
end

def mark_with_comparison()
    if !is_one_dir()
        ret = select_string("Size", ["same", "ignore"], -1)
        
        if ret == -1
            return
        end
        
        size = ret == 0
        
        ret = select_string("Timestamp", ["same", "ignore"], -1)
        
        if ret == -1
            return
        end
        
        timestamp = ret == 0
        
        ret = select_string("Directory", ["ignore", "mark"], -1)
        
        if ret == -1
            return 
        end
        
        directory = ret == 1
        
        i = 0
        while i < cursor_max()
            fname = file_name(i)
            if file_num_sdir(fname) != -1 && fname != "." && fname != ".."
                stat1 = File.stat(adir_path()+fname)
                stat2 = File.stat(sdir_path()+fname)
                
                if (stat1.ftype() == stat2.ftype()) && (!size || size && stat1.size==stat2.size) && (!timestamp || timestamp && stat1.mtime == stat2.mtime) && (directory || !directory && stat1.ftype!="directory")
                    mark_on(i)
                end
            end
            
            i+=1
        end
    end
end

def enter_cursor_down
    if $option_enter_cursor_down
        cursor_move(cursor() + 1)
    end
end

if `uname -s`.index("CYGWIN")
    def key_enter_mark_win_program(prg_name)
        cmdline = 'cmdline2("%Q ' + prg_name;
        adir_mark.each {|fname|
            str = adir_path() + fname
            
            if str.index("'")
               cmd = 'cygpath -l -w "' + str + '"'
            else
               cmd = "cygpath -l -w '" + str + "'"
            end
            out = `#{cmd}`
            out.chomp!()
            
            i = 0
            while i < out.length()
               if (i == 0 || !( (kanjicode()=="eucjp"||kanjicode()=="sjis") && is_kanji(out[i-1]))) && out[i] == '\\'[0]
                   out.insert(i, "\\\\\\")
                   i+=3
               end
               
               i+=1
            end
            
            if out.index("'")
                cmdline = cmdline + ' \\"' + out + '\\"';
            else
                cmdline = cmdline + ' \'' + out + '\'';
            end
        }
        cmdline = cmdline + ' &", 0, nil)';
        
        eval(cmdline);
    end

    def key_enter_mark_win_program2(prg_name)
        cmdline = 'shell("%Q ' + prg_name;
        adir_mark.each {|fname|
            str = adir_path() + fname
            
            if str.index("'")
               cmd = 'cygpath -l -w "' + str + '"'
            else
               cmd = "cygpath -l -w '" + str + "'"
            end
            out = `#{cmd}`
            out.chomp!()
            
            i = 0
            while i < out.length()
               if (i == 0 || !( (kanjicode()=="eucjp"||kanjicode()=="sjis") && is_kanji(out[i-1]))) && out[i] == '\\'[0]
                   out.insert(i, "\\")
                   i+=1
               end
               
               i+=1
            end
            
            if out.index("'")
                cmdline = cmdline + ' \\"' + out + '\\"';
            else
                cmdline = cmdline + ' \'' + out + '\'';
            end
        }
        cmdline = cmdline + ' &", "' + prg_name +  '")';
        
        eval(cmdline);
    end
    
    def key_enter_win_program(prg_name)
        key_enter_mark_win_program(prg_name)
        enter_cursor_down
    end

    def key_enter_win_program2(prg_name)
        key_enter_mark_win_program2(prg_name)
        enter_cursor_down
    end
end

def original_type_pagedown()
    cursor_move(cursor()+$pageup_length)
end

def original_type_pageup()
    cursor_move(cursor()-$pageup_length)
end

def fdclone_type_pagedown()
    if is_one_dir() then
        cursor_move(cursor()+$pageup_length)
    else
        if cursor_max() <= cursor_maxy() then
            cursor_move(cursor()+$pageup_length)
        else
            n = (cursor_max()/cursor_maxy())*cursor_maxy()
            if cursor() < n then
                cursor_move(cursor()+cursor_maxy())
            end
        end
    end
end

def fdclone_type_pageup()
    if is_one_dir() then
        cursor_move(cursor()-$pageup_length)
    else
        if cursor_max() <= cursor_maxy() then
            cursor_move(cursor()-$pageup_length)
        else 
            if cursor() >= cursor_maxy() then
                cursor_move(cursor()-cursor_maxy())
            end
        end
    end
end

def afu_type_pagedown()
    if is_one_dir() then
        cursor_move(cursor()+$pageup_length)
    else
        if cursor_max() <= cursor_maxy() then
            cursor_move(cursor()+$pageup_length)
        else
            cursor_move(cursor()+cursor_maxy()-2)
        end
    end
end

def afu_type_pageup()
    if is_one_dir() then
        cursor_move(cursor()-$pageup_length)
    else
        if cursor_max() <= cursor_maxy() then
            cursor_move(cursor()-$pageup_length)
        else    
            cursor_move(cursor()-cursor_maxy()+2)
        end
    end
end

def pagedown()
    if $option_scroll_type == "afu"
        afu_type_pagedown()
    elsif $option_scroll_type == "original"
        original_type_pagedown()
    elsif $option_scroll_type == "fdclone"
        fdclone_type_pagedown()
    else
        fdclone_type_pagedown()
    end
end

def pageup()
    if $option_scroll_type == "afu"
        afu_type_pageup()
    elsif $option_scroll_type == "original"
        original_type_pageup()
    elsif $option_scroll_type == "fdclone"
        fdclone_type_pageup()
    else
        fdclone_type_pageup()
    end
end

def cursor_up()
    cursor_move(cursor()-1)
end

def cursor_down()
    cursor_move(cursor()+1)
end

def move_left_directory()
    if is_one_dir() then
        if cursor_x() > 0 then
            cursor_move(cursor()-cursor_maxy())
        else
            if $option_left_right_dir_up
                dir_move('..')
            end
        end
    else
        if is_adir_left() then
            if $option_left_right_dir_up
                dir_move('..')
            end
        else
            cursor_left()
        end
    end
end

def move_right_directory()
    if is_one_dir() then
        if cursor_x() < (cursor_maxx()-1)
            cursor_move(cursor()+cursor_maxy())
        end
    else
        if is_adir_right() then
            if $option_left_right_dir_up
                dir_move('..')
            end
        else
            cursor_right()
        end
    end
end

def cursor_top()
    cursor_move(0)
end

def cursor_bottom()
    cursor_move(cursor_max()-1)
end

def move_parent_dir()
    if $sftp_path && is_adir_right
        if $sftp_path != "/"
            fname = $sftp_path.scan(/.*\/(.+)\//)[0][0]
            sftp_change_dir($sftp_path.scan(/(.*\/).+\//)[0][0])
            cursor_move_rdir(file_num_rdir(fname))
        end
    else
        dir_move('..')
    end
end

def move_root_dir()
    if $sftp_path && is_adir_right
        sftp_change_dir("/")
    else
        dir_move('/')
    end
end

def move_home_dir()
    if $sftp_path && is_adir_right
        sftp_change_dir("/home")
    else
        dir_move('~')
    end
end

def sdir_move_home_dir()
    if is_one_dir()
        set_view_filer("all")
        sdir_move('~')
    else
        sdir_move('~')
    end

    cursor_other()
    set_scrolltop(0)
    cursor_move(1)
end

def mark_cursor_file_and_down_cursor()
    mark2(cursor)
    cursor_move(cursor + 1)
end

def toggle_visible_dot_file()
    if adir_mask() == "*"
        $visible_dot_dir = true
        set_glob_mask_hook()
        set_mask("{*,.*}")
        set_mask_sdir("{*,.*}")
        reread()
    else
        $visible_dot_dir = false
        set_glob_mask_hook()
        set_mask("*")
        set_mask_sdir("*")
        reread()
    end

    if $sftp_path
        sftp_path_refresh()
    end
end

def file_copy(path)
    if path[-1] == "/"[0] || marking()
        if !File.exist?(path)
            strings = ["Yes", "No"]
            if select_string("#{path} does not exist. would you like to create this directory?", strings, 1) == 0
                FileUtils.mkdir_p(path)
            end
        end
        if File.directory?(path)
            copy_p(adir_path(), adir_mark(), path, true)
            reread
        end
    else
        copy_p(adir_path(), adir_mark(), path, true)
        reread
    end
end

def copy_cursor_or_mark_file()
    if $sftp_path
        $sftp_override = SFTP_OVERRIDE_NIL

        if is_adir_right
            if $option_afu_copy
                if marking()
                    sftp_get(ldir_path)
                else
                    cmdline2('%r%Q sftp_get("' + ldir_path + '")', -2, nil)
                end
            else
                cmdline2('%r%Q sftp_get("' + ldir_path + '")', -2, nil)
            end
        else
            if $option_afu_copy
                if marking()
                    sftp_put(rdir_path)
                else
                    cmdline2('%r%Q sftp_put("' + rdir_path + '")', -2, nil)
                end
            else
                cmdline2('%r%Q sftp_put("' + rdir_path + '")', -2, nil)
            end
        end
    else
        if is_one_dir()
            cmdline2('%Q%r%M file_copy %D1', -1, nil)
        else 
            if $option_afu_copy
                if marking()
                    copy_p(adir_path(), adir_mark(), sdir_path(), true)
                    reread
                else
                    cmdline2('%Q%r%M file_copy %D2', -1, nil)
                end
            else
                cmdline2('%Q%r%M file_copy %D2', -1, nil)
            end
        end
    end
end

def copy_cursor_or_mark_file2()
    cmdline2('%Q%r%M file_copy %D1', -1, nil)
end

def copy_cursor_or_mark_file3()
    cmdline2('%Q%r%M file_copy %D2', -1, nil)
end

def copy_mark_or_cursor_file_to_tab()
    if tab_max() > 0
        select_str = []

        n = 0
        while n < tab_max()
            select_str.push(tab_path(n))

            n += 1
        end

        select_num = select_menu_mfiler("copy distination", select_str)

        if select_num != -1
            file_copy(select_str[select_num])
        end
    end
end

def copy_mark_or_cursor_file_to_jmenu()
    if $directory_menu.size > 0
        select_str = []
        select_dir = []

        $directory_menu.each() { |item|
            select_str.push(item[0])
            select_dir.push(item[2])
        }

        select_num = select_menu_mfiler("copy distination", select_str)

        if select_num != -1
            file_copy(select_dir[select_num])
        end
    end
end

def copy_mark_or_cursor_file_to_history()
    if $my_dir_history.size > 0
        select_str = []

        $my_dir_history.each() { |item|
            select_str.push(item)
        }

        select_num = select_menu_mfiler("copy distination", select_str)

        if select_num != -1
            file_copy(select_str[select_num])
        end
    end
end

if `uname -s`.chomp() == "FreeBSD" || `uname -s`.chomp() == "Darwin"
    CP_OPTION = "-Rp"
else                        # cygwin || Linux
    CP_OPTION = "-Rpd"
end

def copy_mark_or_cursor_with_cp()
    if is_one_dir()
        cmdline2("cp #{CP_OPTION} %m1 %D1", -1, nil)
    else 
        cmdline2("cp #{CP_OPTION} %m1 %D2", -1, nil)
    end
end

def copy_mark_or_cursor_with_cp_background()
    if is_one_dir()
        cmdline2("%s cp #{CP_OPTION} %m1 %D1 &", -3, nil)
    else 
        cmdline2("%s cp #{CP_OPTION} %m1 %D2 &", -3, nil)
    end
end

defmenu("copy_menu",
    "copy(1)", KEY_1,  "copy_cursor_or_mark_file3()",
    "copy(2)", KEY_2,  "copy_cursor_or_mark_file2()",
    "(h)istory", KEY_h, "copy_mark_or_cursor_file_to_history()",
    "jump (m)enu", KEY_m, "copy_mark_or_cursor_file_to_jmenu()",
    "(t)ab",  KEY_t,  "copy_mark_or_cursor_file_to_tab()",
    "(c)p",  KEY_c,  "copy_mark_or_cursor_with_cp()",
    "(C)p background",  KEY_C,  "copy_mark_or_cursor_with_cp_background()"
)

def file_move(path)
    if path[-1] == "/"[0]
        if !File.exist?(path)
            strings = ["Yes", "No"]
            if select_string("#{path} does not exist. would you like to create this directory?", strings, 1) == 0
                FileUtils.mkdir_p(path)
            end
        end
        
        if File.directory?(path)
            move(adir_path(), adir_mark(), path, true)
            reread
        end
    else
        move(adir_path(), adir_mark(), path, true)
        reread
    end
end

def move_mark_or_cursor_file()
    if $sftp_path
        $sftp_override = SFTP_OVERRIDE_NIL

        if is_adir_right
            if $option_afu_copy
                if marking()
                    sftp_get_and_remove(ldir_path)
                else
                    cmdline2('%r%Q sftp_get_and_remove("' + ldir_path + '")', -2, nil)
                end
            else
                cmdline2('%r%Q sftp_get_and_remove("' + ldir_path + '")', -2, nil)
            end
        else
            if $option_afu_copy
                if marking()
                    sftp_put_and_remove()
                else
                    cmdline2('%r%Q sftp_put_and_remove()', -2, nil)
                end
            else
                cmdline2('%r%Q sftp_put_and_remove()', -2, nil)
            end
        end
    else
        if is_one_dir() 
            cmdline2('%Q%r%M file_move %D1', -1, nil)
        else 
            if $option_afu_copy
                if marking()
                    move(adir_path(), adir_mark(), sdir_path(), true)
                    reread
                else
                    cmdline2('%Q%r%M file_move %D2', -1, nil)
                end
            else
                cmdline2('%Q%r%M file_move %D2', -1, nil)
            end
        end
    end
end

def move_mark_or_cursor_file2()
    cmdline2('%Q%r%M file_move %D1', -1, nil)
end

def move_mark_or_cursor_file_to_tab()
    if tab_max() > 0
        select_str = []

        n = 0
        while n < tab_max()
            select_str.push(tab_path(n))

            n += 1
        end

        select_num = select_menu_mfiler("move distination", select_str)

        if select_num != -1
            file_move select_str[select_num]
        end
    end
end

def move_mark_or_cursor_file_to_jmenu()
    if $directory_menu.size > 0
        select_str = []
        select_dir = []

        $directory_menu.each() { |item|
            select_str.push(item[0])
            select_dir.push(item[2])
        }

        select_num = select_menu_mfiler("move distination", select_str)

        if select_num != -1
            file_move select_dir[select_num]
        end
    end
end

def move_mark_or_cursor_file_to_history()
    if $my_dir_history.size > 0
        select_str = []

        $my_dir_history.each() { |item|
            select_str.push(item)
        }

        select_num = select_menu_mfiler("move distination", select_str)

        if select_num != -1
            file_move select_str[select_num]
        end
    end
end

def move_mark_or_cursor_file_with_mv()
    if is_one_dir()
        cmdline2("mv %m1 %D1", -1, nil)
    else 
        cmdline2("mv %m1 %D2", -1, nil)
    end
end

def move_mark_or_cursor_file_with_mv_background()
    if is_one_dir()
        cmdline2("%s mv %m1 %D1 &", -3, nil)
    else 
        cmdline2("%s mv %m1 %D2 &", -3, nil)
    end
end

def key_move_history()
    move_mark_or_cursor_file_to_history()
end

def key_move_jmenu()
    move_mark_or_cursor_file_to_jmenu()
end

def key_move_tab()
    move_mark_or_cursor_file_to_tab()
end

def key_move_mv()
    move_mark_or_cursor_file_with_mv()
end

defmenu("move_menu",
    "move(1)", KEY_1,  "move_mark_or_cursor_file()",
    "move(2)", KEY_2,  "move_mark_or_cursor_file2()",
    "(h)istory", KEY_h, "move_mark_or_cursor_file_to_history()",
    "jump (m)enu", KEY_m, "move_mark_or_cursor_file_to_jmenu()",
    "(t)ab",  KEY_t,  "move_mark_or_cursor_file_to_tab()",
    "(m)v",  KEY_m,  "move_mark_or_cursor_file_with_mv()",
    "(M)v background",  KEY_M,  "move_mark_or_cursor_file_with_mv_background()"
)

def delete_mark_or_cursor_file()
    if $sftp_path && is_adir_right
        ret = select_string("remove ok?", ["Yes", "No"], 1)
        if ret == 0
            if $sftp_passwd
                Net::SFTP.start($sftp_host, $sftp_user, $sftp_passwd) do |sftp|
                    rdir_mark().each do |file|
                        kind = file_kind_rdir(file_num_rdir(file))

                        if kind == 1    # ディレクトリ
                            sftp_rmdir_r(sftp, $sftp_path + file + "/")
                        else
                            sftp.remove($sftp_path + file)
                        end

                        mark_off(file_num(file))
                    end
                end
            else
                Net::SFTP.start($sftp_host, $sftp_user, :host_key=>"ssh-rsa") do |sftp|
                    rdir_mark().each do |file|
                        kind = file_kind_rdir(file_num_rdir(file))

                        if kind == 1    # ディレクトリ
                            sftp_rmdir_r(sftp, $sftp_path + file + "/")
                        else
                            sftp.remove($sftp_path + file)
                        end

                        mark_off(file_num(file))
                    end
                end
            end

            reread_rdir()
        end
    else
        remove(adir_path(), adir_mark(), true)
        reread_adir()
    end
end

def throw_mark_or_cursor_file()
    if $sftp_path && is_adir_right
        ret = select_string("remove ok?", ["Yes", "No"], 1)
        if ret == 0
            if $sftp_passwd
                Net::SFTP.start($sftp_host, $sftp_user, $sftp_passwd) do |sftp|
                    rdir_mark().each do |file|
                        kind = file_kind_rdir(file_num_rdir(file))

                        if kind == 1    # ディレクトリ
                            sftp_rmdir_r(sftp, $sftp_path + file + "/")
                        else
                            sftp.remove($sftp_path + file)
                        end

                        mark_off(file_num(file))
                    end
                end
            else
                Net::SFTP.start($sftp_host, $sftp_user, :host_key=>"ssh-rsa") do |sftp|
                    rdir_mark().each do |file|
                        kind = file_kind_rdir(file_num_rdir(file))

                        if kind == 1    # ディレクトリ
                            sftp_rmdir_r(sftp, $sftp_path + file + "/")
                        else
                            sftp.remove($sftp_path + file)
                        end

                        mark_off(file_num(file))
                    end
                end
            end

            reread_rdir()
        end
    else
        trashbox(adir_path(), adir_mark(), true)
        reread_adir()
    end
end

def delete_mark_or_cursor_file_with_rm()
    cmdline2('rm -R %m1', 0, nil)
end

def delete_mark_or_cursor_file_with_rm_background()
    cmdline2('%s rm -R %m1 &', 0, nil)
end

defmenu('delete_menu',
    "(t)rashbox", KEY_t, "throw_mark_or_cursor_file()",
    "(d)elete", KEY_d, "delete_mark_or_cursor_file()",
    "(r)m",     KEY_r, "delete_mark_or_cursor_file_with_rm()",
    "(R)m background", KEY_R, "delete_mark_or_cursor_file_with_rm_background()"
)

def edit_cursor_file()
    shell('%q $EDITOR %f', cursor_name_convert)
end

def edit_new_file()
    cmdline2('%q $EDITOR ""', -1, nil)
end

def view_cursor_file()
    if !marking
        shell('%q $PAGER %f', cursor_name_convert)
    end
end

def rename(cname, fname)
    if File.exist?(adir_path() + fname)
        message("file exists(" + fname + ")")
        return false
    end

    begin
        File.rename(cname, fname)
    rescue
        message("rename failed")
        return false
    end
    reread_adir()
    cursor_move(file_num(fname))
    
    return true
end

def rename_cursor_or_mark_file()
    if marking()
        rename_mark_file()
    else
        rename_cursor_file()
    end
end

def rename_cursor_file()
    if !marking()
        p = 0
        if cursor_name()[0] != "."[0]
            ret = cursor_name().rindex(".")
            if ret
                p = cursor_name().length - ret
            end
        end
        
        cmdline2('%Q%r rename %f, %f', -1 -p, nil)
    end
end

def rename2(fname)
    if File.exist?(adir_path() + fname)
        message("file exists(" + fname + ")")
        return false
    end

    begin
        File.rename(cursor_name(), fname)
    rescue
        message("rename failed")
        return false
    end

    reread_adir()
    cursor_move(file_num(fname))
    
    return true
end

def rename_cursor_file2()
    if !marking()
        cmdline2('%Q%r rename2 ""', -1, nil)
    end
end

def rename_mark_file()
    cmdline2("%r%Q gsub_rename //, \"\"", -5, nil)
end

def gsub_rename(from, to)
=begin
    ret1 = input_box("from", "", 0)
    if ret1[0] == 1
        return
    end
    from = ret1[1]
    
    ret2 = input_box("to", "", 0)
    if ret2[0] == 1
        return
    end
    to = ret2[1]
=end
    
    adir_mark2 = []
    adir_mark().each {|file|
        begin
            file2 = file.gsub(from, to)
        rescue
            message("regex error")
            mclear_immediately()
            return
        end
        
        adir_mark2.push(file2)
    }
    
    ### kakuninn ###
    maxx = mgetmaxx()
    maxy = mgetmaxy()
    
    i = 0
    while i < adir_mark2.size() 
        maxx = mgetmaxx()
        maxy = mgetmaxy()
        
        mclear()
        mfiler_view()
        
        mbox(0, 0, maxx-1, maxy-2)
        line = 0
        while line*2+2 < maxy-2 && i < adir_mark2.size()
            mmvprintw(line*2+1, 1, adir_mark()[i])
            mmvprintw(line*2+2, 1, ' -> ' + adir_mark2[i])
            line+=1
            i+=1
        end
        
        if i < adir_mark2.size()
            message("HIT ANY KEY")
        end
    end
    
    ret3 = select_string("rename ok?", ["Yes", "No"], 1)
    
    if ret3 == 1
        return
    end
    
    i = 0
    adir_mark_ = adir_mark().clone
    while i < adir_mark_.size()
        fnum = file_num(adir_mark_[i])
        if adir_mark_[i] != adir_mark2[i] 
            if !rename(adir_mark_[i], adir_mark2[i])
                break
            end
        end
        
        mark_off(fnum)
        i+=1
    end
    
    reread_adir
end

def backup_mark_file()
    adir_mark().each do |fname| 
        result = select_string("Would you like to make " + fname + ".bak?", ["Yes", "No"], -1)
        if result == 0
            copy_p(adir_path(), [fname], fname + ".bak", true)
        end
    end

    reread_adir()
end

def backup_cursor_file()
    if cursor_name().index("'")
        cmdline2("%Q%r copy_p(adir_path(), adir_mark(), \"#{cursor_name_convert()}.bak\", true); reread", -16, nil)
    else
        cmdline2("%Q%r copy_p(adir_path(), adir_mark(), '#{cursor_name_convert()}.bak', true); reread", -16, nil)
    end
end

def backup_cursor_or_mark_file()
    if marking()
        backup_mark_file()
    else
        backup_cursor_file()
    end
end

def pack_mark_or_cursor_file()
    if is_one_dir()
        if marking()
            if adir_path().index("'")
                cmdline2("tar cvfz \"#{adir_path_convert()}.tgz\" %m1", -9, nil)
            else
                cmdline2("tar cvfz '#{adir_path_convert()}.tgz' %m1", -9, nil)
            end
        else
            if adir_path().index("'") || File.basename(cursor_noext()).index("'")
                cmdline2("tar cvfz \"#{adir_path_convert() + File.basename(cursor_noext_convert())}.tgz\" %m1", -9, nil)
            else
                cmdline2("tar cvfz '#{adir_path_convert() + File.basename(cursor_noext_convert())}.tgz' %m1", -9, nil)
            end
        end
    else
        if marking()
            if sdir_path().index("'")
                cmdline2("tar cvfz \"#{sdir_path_convert()}.tgz\" %m1", -9, nil)
            else
                cmdline2("tar cvfz '#{sdir_path_convert()}.tgz' %m1", -9, nil)
            end
        else
            if sdir_path().index("'") || File.basename(cursor_noext()).index("'")
                cmdline2("tar cvfz \"#{sdir_path_convert() + File.basename(cursor_noext_convert())}.tgz\" %m1", -9, nil)
            else
                cmdline2("tar cvfz '#{sdir_path_convert() + File.basename(cursor_noext_convert())}.tgz' %m1", -9, nil)
            end
        end
    end
end

def pack_mark_or_cursor_file2()
    if marking()
        cmdline2("tar cvfz '.tgz' %m1", -9, nil)
    else
        if File.basename(cursor_noext()).index("'")
            cmdline2("tar cvfz \"#{File.basename(cursor_noext_convert())}.tgz\" %m1", -9, nil)
        else
            cmdline2("tar cvfz '#{File.basename(cursor_noext_convert())}.tgz' %m1", -9, nil)
        end
    end
end

def make_dir(fname)
    if $sftp_path && is_adir_right
        if $sftp_passwd == nil
            Net::SFTP.start($sftp_host, $sftp_user, :host_key=>"ssh-rsa") do |sftp|
                sftp.mkdir($sftp_path + fname, :permissions=>0755)
            end
        else
            Net::SFTP.start($sftp_host, $sftp_user, $sftp_passwd) do |sftp|
                sftp.mkdir($sftp_path + fname, :permissions=>0755)
            end
        end

        reread_rdir()
    else
        if File.exist?(adir_path() + fname)
            message("file exists")
            return
        end

        begin
            #Dir.mkdir(adir_path() + fname, 0755)
            FileUtils.mkdir_p(adir_path() + fname)
        rescue
            message("make_dir() failed")
        end

        reread_adir()
        if fname.index("/")
            cursor_move(file_num(fname[0..fname.index("/")-1]))
        else
            cursor_move(file_num(fname))
        end
    end
end

def make_directory()
   cmdline_noconvert("%r%Q make_dir ''", -1, nil)
end

def make_new_file(fname)
    if $sftp_path && is_adir_right
        if $sftp_passwd == nil
            Net::SFTP.start($sftp_host, $sftp_user, :host_key=>"ssh-rsa") do |sftp|
                f = open("/tmp/" + fname, "w")
                f.close 

                sftp.put_file("/tmp/" + fname, $sftp_path + fname)
            end
        else
            Net::SFTP.start($sftp_host, $sftp_user, $sftp_passwd) do |sftp|
                f = open("/tmp/" + fname, "w")
                f.close 

                sftp.put_file("/tmp/" + fname, $sftp_path + fname)
            end
        end

        reread_rdir()
    else
        if File.exist?(adir_path() + fname)
            message("file exists")
            return
        end

        begin 
            f = File.new(adir_path() + fname,File::CREAT|File::TRUNC|File::WRONLY)
            #f.chmod(0644)
            f.close()
        rescue
            message("make_new_file() failed")
        end
        reread_adir()
        cursor_move(file_num(fname))
    end
end

def make_new_file_cmdline()
    cmdline_noconvert("%r%Q make_new_file ''", -1, nil)
end

def make_new_link(src, dest)
    if File.exist?(dest) && File.stat(dest).directory?
        if dest[dest.length-1] != "/"[0]
            dest = dest + "/"
        end

        if src =~ /\/\/.+\/.+/
            bname = src.scan(/\/\/.+\/(.+)/)[0][0]
        else
            bname= File.basename(src)
        end

        dest = dest + bname
    end

    if File.exist?(dest)
        message("file exists")
        return
    end

    begin 
        File.symlink(src, dest)
    rescue
        message("make_new_link() failed")
    end
end

def make_new_link_mark(src, dest)
    if !File.exist?(dest) || !File.stat(dest).directory?
        message("distination is not directory")
        return
    end

    if dest[dest.length-1] != "/"[0]
        dest = dest + "/"
    end

    begin
        mark_num_ = mark_num()
        progress_mark = mark_num()
        
        src.each {|path|
            if path =~ /\/\/.+\/.+/
                fname = path.scan(/\/\/.+\/(.+)/)[0][0]
            else
                fname = File.basename(path)
            end
            
            cursor_move(file_num(fname))
            
            ### draw progress box ###
            mfiler_view()
            
            maxx = mgetmaxx()
            maxy = mgetmaxy()
            
            y = maxy / 2
            
            mbox(y, (maxx-22)/2, 22, 3)
            mmvprintw(y, (maxx-22)/2+2, "progress")
            mmvprintw(y+1, (maxx-22)/2+1, "(%d/%d) files", progress_mark, mark_num_)
            mrefresh()
            
            ### make sym link ###
            File.symlink(path, dest + fname)
            
            fname2 = path[adir_path().length..-1]
            mark_off(file_num(fname2))
            progress_mark-=1
        }
    rescue
        message("make_new_link_mark() failed")
    end
    
    mfiler_view()
    mrefresh()
end

def make_link()
    if is_one_dir() 
        if marking()
            cmdline2("%r%Q%Mmake_new_link_mark(adir_mark_fullpath(), %D1); reread", -10, nil);
        else
            cmdline2("%r%Q%Mmake_new_link(%F, %D1); reread", -10, nil)
        end
    else 
        if $option_afu_copy
            if marking()
                make_new_link_mark(adir_mark_fullpath(), sdir_path())
                reread
            end
        else
            if marking()
               cmdline2('%r%Q%Mmake_new_link_mark(adir_mark_fullpath(), %D2); reread', -10, nil)
            else
                cmdline2("%r%Q%Mmake_new_link(%F, %D2); reread", -10, nil)
            end
        end
    end
end

defmenu("osx_make_link",
    "make (s)ymbolic link in current directory", KEY_s, "make_link2",
    "make (a)lias", KEY_a, "make_osx_alias"
)

def mkalias(fname, dist)
    if dist[-1] != "/"[0]
        dist = dist + "/"
    end

    shell("%q mkalias \"#{fname}\" \"#{dist}#{fname}\"", fname)
end

def mkalias_marking(dist)
    if dist[-1] != "/"[0]
        dist = dist + "/"
    end

    adir_mark().each do |fname|
        shell("%q mkalias \"#{fname}\" \"#{dist}#{fname}\"", fname)
    end
end

def make_osx_alias()
    if is_one_dir()
        if marking()
            cmdline2("%r%q mkalias_marking %D1", -1, nil)
        else
            cmdline2("%r%q mkalias %f, %D1", -1, nil)
        end
    else
        if $option_afu_copy
            if marking()
                mkalias_marking sdir_path
            end
        else
            if marking()
                cmdline2("%r%q mkalias_marking %D2", -1, nil)
            else
                cmdline2("%r%q mkalias %f, %D2", -1, nil)
            end
        end
    end
end

def make_link2()
    if !marking()
        cmdline2("%r%Qmake_new_link(%F, ''); reread", -10, nil)
    end
end

def make_link_menu()
    if `uname -s`.index("Darwin")
        menu("osx_make_link")
    else
        make_link2()
    end
end

def unpack_gz
    if cursor_name().rindex(".tar.gz")
        if is_one_dir() 
            cmdline2('tar xvfz %f -C %D1', -1, nil)
        else
            cmdline2('tar xvfz %f -C %D2', -1, nil)
        end
    else
        cmdline2('gzip -d %F', -1, nil)
    end
end

def unpack_gz2()
    if cursor_name().rindex(".tar.gz")
        cmdline2('tar xvfz %f -C %D1', -1, nil)
    else
        cmdline2('gzip -d %F', -1, nil)
    end
end

def unpack_bz2()
    if cursor_name().rindex(".tar.bz2")
        if is_one_dir()
            cmdline2('tar xvfj %f -C %D1', -1, nil)
        else
            cmdline2('tar xvfj %f -C %D2', -1, nil)
        end
    else
        cmdline2('bzip2 -d %f', -1, nil)
    end
end

def unpack_bz22()
    if cursor_name().rindex(".tar.bz2")
        cmdline2('tar xvfj %f -C %D1', -1, nil)
    else
        cmdline2('bzip2 -d %f', -1, nil)
    end
end

def unpack_tar()
    if is_one_dir()
        cmdline2('tar xvf %f -C %D1', -1, nil)
    else
        cmdline2('tar xvf %f -C %D2', -1, nil)
    end
end

def unpack_tar2()
    cmdline2('tar xvf %f -C %D1', -1, nil)
end

def unpack_tgz()
    if is_one_dir()
        cmdline2('tar xvfz %f -C %D1', -1, nil)
    else
        cmdline2('tar xvfz %f -C %D2', -1, nil)
    end
end

def unpack_tgz2()
    cmdline2('tar xvfz %f -C %D1', -1, nil)
end

def unpack_lzh_main(fname, directory)
    cwd = Dir.pwd
    Dir.chdir(directory)
    
    shell('lha x ' + fname, "lha")
    
    Dir.chdir(cwd)
end

def unpack_lzh_main_sudo(fname, directory)
    cwd = Dir.pwd
    Dir.chdir(directory)
    
    shell('sudo lha x ' + fname, "lha")
    
    Dir.chdir(cwd)
end

def unpack_lzh()
    if is_one_dir()
        cmdline2('%S cd %D1; lha x %F', adir_path().size() + 8, nil)
    else
        cmdline2('%S cd %D2; lha x %F', sdir_path().size() + 8, nil)
    end
end

def unpack_lzh2()
    cmdline2('%S cd %D1; lha x %F', adir_path().size() + 8, nil)
end

def unpack_zip()
    if is_one_dir()
        cmdline2('unzip %f -d %D1', -1, nil)
    else
        cmdline2('unzip %f -d %D2', -1, nil)
    end
end

def unpack_zip2()
    cmdline2('unzip %f -d %D1', -1, nil)
end

def glob_star_nodir()
    cmd = ""
    
    Dir.glob("*").each do |fname|
        if !File.directory?(fname)
            cmd = cmd + fname + " "
        end
    end
    
    cmd = cmd[0..-2]
    
    return cmd
end

def glob_star()
    cmd = ""
    
    Dir.glob("*").each do |fname|
        if fname != "." && fname != ".."
            cmd = cmd + fname + " "
        end
    end
    
    cmd = cmd[0..-2]
    
    return cmd
end

def grep_current_directory()
    if marking()
        cmdline2("%q grep '' %m1 -n 2> /dev/null | $PAGER", 10, nil)
    else
        cmdline2("%q%S grep '' * -n 2> /dev/null | $PAGER", 12, nil)
    end
end

def grep_recursive()
    if marking()
        if `uname -s`.chomp() == "Linux" || `uname -s`.index("CYGWIN") || `uname -s`.index("Darwin")
            cmdline2("%q grep -R '' %m1 -n 2> /dev/null | $PAGER", 13, nil)
        else
            cmdline2("%q find . | xargs grep -n '' 2> /dev/null | $PAGER", 28, nil)
        end
    else
        if `uname -s`.chomp() == "Linux" || `uname -s`.index("CYGWIN") || `uname -s`.index("Darwin")
            cmdline2("%q%S grep -R '' * -n 2> /dev/null | $PAGER", 15, nil)
        else
            cmdline2("%q find . | xargs grep -n '' 2> /dev/null | $PAGER", 28, nil)
        end
    end
end

def mgrep_current_directory()
    if $sftp_path && is_adir_right
    else
        cmdline2("%Q%r mgrep \"grep -n '' * 2>&1 | tee /tmp/mfiler_out_of_grep\", ENV['EDITOR'], '+'", 22, nil)
    end
end

def mgrep_recursive()
    cmdline2("%Q%r mgrep \"grep -R -n '' * 2>&1 | tee /tmp/mfiler_out_of_grep\", ENV['EDITOR'], '+'", 25, nil)
end

def sed_current_directory()
    cmdline2("%S for i in *; do sed 's///g' $i 2> /dev/null 1> /tmp/$i; cp -f /tmp/$i $i; done", 26, nil)
end

def sed_mark_file()
    cmdline2("%S for i in %m1; do sed 's///g' $i 2> /dev/null 1> /tmp/$i; cp -f /tmp/$i $i; done", 28, nil)
end

def info_directory()
    cmdline2('du -sh %f', 7, nil)
end

def info_file()
    cmdline2('%r puts `file %f`; print "file name encode: "; puts encode_type(cursor_name)', 0, nil)
end

def open_dir_in_other()
    if is_one_dir()
        if is_adir_left()
            sdir_move(adir_path() + cursor_name())
            set_view_filer("all")
            cursor_other()
        else
            sdir_move(adir_path())
            cursor_move_sdir(cursor())
            set_scrolltop_sdir(scrolltop())

            dir_move(adir_path() + cursor_name())
            set_view_filer("all")
        end
    else
        sdir_move(adir_path() + cursor_name())
    end
end

def copy_path()
    if is_one_dir()
        if File.directory?(adir_path + cursor_name)
            open_dir_in_other()
        end
    else
        if !is_one_dir()
            dir_move(sdir_path())
            cursor_move(cursor_sdir())
            set_scrolltop(scrolltop_sdir())
            set_sort_kind(adir_num(), sort_kind(sdir_num()))
            set_sort_dir_up(adir_num(), sort_dir_up(sdir_num()))
            set_view_nameonly(adir_num(), view_nameonly(sdir_num()))
            set_view_focusback(adir_num(), view_focusback(sdir_num()))
            set_view_removedir(adir_num(), view_removedir(sdir_num()))
            
            if is_adir_left() && $out_of_find_rdir != nil
                $out_of_find_ldir = $out_of_find_rdir
                
                reread()
            elsif is_adir_right() && $out_of_find_ldir != nil
                $out_of_find_rdir = $out_of_find_ldir
                
                reread()
            end
            
            if is_adir_left() && $out_of_locate_rdir != nil
                $out_of_locate_ldir = $out_of_locate_rdir
                
                reread()
            elsif is_adir_right() && $out_of_locate_ldir != nil
                $out_of_locate_rdir = $out_of_locate_ldir
                
                reread()
            end
            
            if is_adir_left() && $out_of_grep_rdir != nil
                $out_of_grep_ldir = $out_of_grep_rdir
                
                reread()
            elsif is_adir_right() && $out_of_grep_ldir != nil
                $out_of_grep_rdir = $out_of_grep_ldir
                
                reread()
            end

            if is_adir_left() && $playlist_rdir != nil
                $playlist_ldir = $playlist_rdir
                $playlist_pwd_ldir = $playlist_pwd_rdir
                $playlist_scrolltop_ldir = $playlist_scrolltop_rdir
                $playlist_cursor_ldir = $playlist_cursor_rdir
                $playlist_focusback_ldir = $playlist_focusback_rdir
                reread()
            elsif is_adir_right() && $playlist_ldir != nil
                $playlist_rdir = $playlist_ldir
                $playlist_pwd_rdir = $playlist_pwd_ldir
                $playlist_scrolltop_rdir = $playlist_scrolltop_ldir
                $playlist_cursor_rdir = $playlist_cursor_ldir
                $playlist_focusback_rdir = $playlist_focusback_ldir

                reread()
            end
        end
    end
end

def copy_path2()
    if !is_one_dir()
        sdir_move(adir_path())
        cursor_move_sdir(cursor())
        set_scrolltop_sdir(scrolltop())
        set_sort_kind(adir_num(), sort_kind(sdir_num()))
        set_sort_dir_up(adir_num(), sort_dir_up(sdir_num()))
        set_view_nameonly(adir_num(), view_nameonly(sdir_num()))
        set_view_focusback(adir_num(), view_focusback(sdir_num()))
        set_view_removedir(adir_num(), view_removedir(sdir_num()))
        
        if is_adir_left() && $out_of_find_ldir != nil
            $out_of_find_rdir = $out_of_find_ldir
            
            reread()
        elsif is_adir_right() && $out_of_find_rdir != nil
            $out_of_find_ldir = $out_of_find_rdir
            
            reread()
        end
        
        if is_adir_left() && $out_of_locate_ldir != nil
            $out_of_locate_rdir = $out_of_locate_ldir
            
            reread()
        elsif is_adir_right() && $out_of_locate_rdir != nil
            $out_of_locate_ldir = $out_of_locate_rdir
            
            reread()
        end
        
        if is_adir_left() && $out_of_grep_ldir != nil
            $out_of_grep_rdir = $out_of_grep_ldir
            
            reread()
        elsif is_adir_right() && $out_of_grep_rdir != nil
            $out_of_grep_ldir = $out_of_grep_rdir
            
            reread()
        end
        
        if is_adir_left() && $playlist_ldir != nil
            $playlist_rdir = $playlist_ldir
            $playlist_pwd_rdir = $playlist_pwd_ldir
            $playlist_scrolltop_rdir = $playlist_scrolltop_ldir
            $playlist_cursor_rdir = $playlist_cursor_ldir
            $playlist_focusback_rdir = $playlist_focusback_ldir
            
            reread()
        elsif is_adir_right() && $playlist_rdir != nil
            $playlist_ldir = $playlist_rdir
            $playlist_pwd_ldir = $playlist_pwd_rdir
            $playlist_scrolltop_ldir = $playlist_scrolltop_rdir
            $playlist_cursor_ldir = $playlist_cursor_rdir
            $playlist_focusback_ldir = $playlist_focusback_rdir
            
            reread()
        end
    end
end

def exchange_path()
    if !is_one_dir()
        out_ldir = $out_of_find_ldir
        out_rdir = $out_of_find_rdir

        out_of_grep_ldir = $out_of_grep_ldir
        out_of_grep_rdir = $out_of_grep_rdir

        out_of_locate_ldir = $out_of_locate_ldir
        out_of_locate_rdir = $out_of_locate_rdir

        list_ldir = $playlist_ldir
        list_pwd_ldir = $playlist_pwd_ldir
        list_scrolltop_ldir = $playlist_scrolltop_ldir
        list_cursor_ldir = $playlist_cursor_ldir
        list_focusback_ldir = $playlist_focusback_ldir
        list_rdir = $playlist_rdir
        list_pwd_rdir = $playlist_pwd_rdir
        list_scrolltop_rdir = $playlist_scrolltop_rdir
        list_cursor_rdir = $playlist_cursor_rdir
        list_focusback_rdir = $playlist_focusback_rdir

        d = adir_path()
        c = cursor()
        s = scrolltop()
        sk = sort_kind(adir_num())
        sdu = sort_dir_up(adir_num())
        vno = view_nameonly(adir_num())
        vfb = view_focusback(adir_num())
        vrd = view_removedir(adir_num())
        
        dir_move(sdir_path())
        cursor_move(cursor_sdir())
        set_scrolltop(scrolltop_sdir())
        set_sort_kind(adir_num(), sort_kind(sdir_num()))
        set_sort_dir_up(adir_num(), sort_dir_up(sdir_num()))
        set_view_nameonly(adir_num(), view_nameonly(sdir_num()))
        set_view_focusback(adir_num(), view_focusback(sdir_num()))
        set_view_removedir(adir_num(), view_removedir(sdir_num()))
        
        sdir_move(d)
        cursor_move_sdir(c)
        set_scrolltop_sdir(s)
        set_sort_kind(sdir_num(), sk)
        set_sort_dir_up(sdir_num(), sdu)
        set_view_nameonly(sdir_num(), vno)
        set_view_focusback(sdir_num(), vfb)
        set_view_removedir(sdir_num(), vrd)

        cursor_other()
        
        $out_of_find_ldir = out_rdir
        $out_of_find_rdir = out_ldir
        if $out_of_find_ldir != nil || $out_of_find_rdir != nil
            reread()
        end
        
        $out_of_grep_ldir = out_of_grep_rdir
        $out_of_grep_rdir = out_of_grep_ldir
        if $out_of_grep_ldir != nil || $out_of_grep_rdir != nil
            reread()
        end
        
        $out_of_locate_ldir = out_of_locate_rdir
        $out_of_locate_rdir = out_of_locate_ldir
        if $out_of_locate_ldir != nil || $out_of_locate_rdir != nil
            reread()
        end

        $playlist_ldir = list_rdir
        $playlist_pwd_ldir = list_pwd_rdir
        $playlist_scrolltop_ldir = list_scrolltop_rdir
        $playlist_cursor_ldir = list_cursor_rdir
        $playlist_focusback_ldir = list_focusback_rdir

        $playlist_rdir = list_ldir
        $playlist_pwd_rdir = list_pwd_ldir
        $playlist_scrolltop_rdir = list_scrolltop_ldir
        $playlist_cursor_rdir = list_cursor_ldir
        $playlist_focusback_rdir = list_focusback_ldir

        if $playlist_ldir != nil || $playlist_rdir != nil
            reread()
        end
    end
end

def mvgrep(cmd)
    puts "grepping disk..."
    
    set_signal_clear()
    if system(cmd)
        set_signal_mfiler2()

        f = open("/tmp/mfiler_out_of_grep2", "r")
        out = f.read()
        f.close()

        if is_adir_left()
            $out_of_grep_ldir = out
        else
            $out_of_grep_rdir = out
        end

        reread()
    else
        set_signal_mfiler2()
    end
end


def disk_read_hook_ldir()
    if $out_of_find_ldir
        dir = Dir.pwd()
        Dir.chdir(ldir_path())
        
        find_out = $out_of_find_ldir
    
        add_file_ldir("..")
        
        find_out.each do |fname|
            fname.chomp!()
            
            if fname == "."
                next
            end
            
            if fname[0..1] == "./"
                fname = fname[2..-1]
            end
            
            add_file_ldir(fname)
        end
        
        Dir.chdir(dir)
        
        return true
    elsif $out_of_locate_ldir
        locate_out = $out_of_locate_ldir
        
        add_file_ldir("..")
        
        locate_out.each do |fname|
            fname.chomp!()
            
            if fname == "."
                next
            end
            
            add_file_ldir(fname)
        end
        
        return true
    elsif $out_of_grep_ldir
        dir = Dir.pwd()
        Dir.chdir(ldir_path())
        
        find_out = $out_of_grep_ldir
    
        add_file_ldir("..")
        
        find_out.each do |line|
            line.chomp!()

            a = line.scan /(^.+):/
            if a
                b = a[0]

                if b
                    c = b[0]
                end
            end

            fname = c
            
            if fname 
                add_file_ldir(fname)
            end
        end
        
        Dir.chdir(dir)
        
        return true
    elsif $playlist_ldir
        add_file_ldir("..")

        $playlist_ldir.each() do |fname|
            fname.chomp!()

            if fname[0] == "/"[0]
                fname = fname[1..-1]
            end

            add_file_ldir(fname)
        end

        return true
    else
        return nil
    end
end

def disk_read_hook_rdir()
    if $out_of_find_rdir
        dir = Dir.pwd()
        Dir.chdir(rdir_path())
        
        find_out = $out_of_find_rdir
        
        add_file_rdir("..")
        
        find_out.each { |fname|
            fname.chomp!()
            
            if fname == "."
                next
            end
            
            if fname[0..1] == "./"
                fname = fname[2..-1]
            end
            
            add_file_rdir(fname)
        }
        
        Dir.chdir(dir)
        
        return true
    elsif $out_of_locate_rdir
        locate_out = $out_of_locate_rdir
        
        add_file_rdir("..")
        
        locate_out.each do |fname|
            fname.chomp!()
            
            if fname == "."
                next
            end
            
            add_file_rdir(fname)
        end
        
        return true
    elsif $sftp_path
        sftp_disk_read_hook_rdir()
    elsif $out_of_grep_rdir
        dir = Dir.pwd()
        Dir.chdir(rdir_path())
        
        find_out = $out_of_grep_rdir
    
        add_file_rdir("..")
        
        find_out.each do |line|
            line.chomp!()

            a = line.scan /(^.+):/
            if a
                b = a[0]

                if b
                    c = b[0]
                end
            end

            fname = c
            
            if fname 
                add_file_rdir(fname)
            end
        end
        
        Dir.chdir(dir)
        
        return true
    elsif $playlist_rdir
        add_file_rdir("..")

        $playlist_rdir.each() do |fname|
            fname.chomp!()

            if fname[0] == "/"[0]
                fname = fname[1..-1]
            end

            add_file_rdir(fname)
        end

        return true
    else
        return nil
    end
end

def key_escape()
    if $playlist_ldir || $playlist_rdir || $out_of_find_ldir || $out_of_find_rdir || $out_of_grep_ldir || $out_of_grep_rdir || $out_of_locate_ldir || $out_of_locate_rdir
        finish_virtual_directory()
    end
end

def finish_virtual_directory()
    if is_adir_left
        if $playlist_ldir
            if $playlist_pwd_ldir != adir_path()
                dir_move_ldir($playlist_pwd_ldir)
            end
            playlist_end_hook_ldir()

            $playlist_ldir = nil

            reread()

            set_scrolltop_ldir($playlist_scrolltop_ldir)
            cursor_move_ldir($playlist_cursor_ldir)
        else
            if $out_of_locate_ldir
                dir_move_ldir($locate_path_before_ldir)
                set_view_focusback(0, $locate_focusback_ldir)
                toggle_nameonly_mode()
            end
            $out_of_locate_ldir = nil
            $out_of_find_ldir = nil
            $out_of_grep_ldir = nil

            reread()
        end
    else
        if $playlist_rdir
            if $playlist_pwd_rdir != adir_path()
                dir_move_rdir($playlist_pwd_rdir)
            end
            playlist_end_hook_rdir()

            $playlist_rdir = nil

            reread()

            set_scrolltop_rdir($playlist_scrolltop_rdir)
            cursor_move_rdir($playlist_cursor_rdir)
        else
            if $out_of_locate_rdir
                dir_move_rdir($locate_path_before_rdir)
                set_view_focusback(1, $locate_focusback_rdir)
                toggle_nameonly_mode()
            end
            $out_of_locate_rdir = nil
            $out_of_find_rdir = nil
            $out_of_grep_rdir = nil

            reread()
        end
    end
end

def path_change_hook(is_ldir)
    if is_ldir
        if $playlist_ldir
            playlist_end_hook_ldir()
        end
        $playlist_ldir = nil

        $out_of_locate_ldir = nil
        $out_of_find_ldir = nil
        $out_of_grep_ldir = nil
    else
        if $playlist_rdir
            playlist_end_hook_rdir()
        end
        $playlist_rdir = nil

        $out_of_locate_rdir = nil
        $out_of_find_rdir = nil
        $out_of_grep_rdir = nil
    end
end

def mfind(cmd)
    puts "searching disk..."
    
    set_signal_clear()
    if system(cmd)
        set_signal_mfiler2()

        f = open("/tmp/mfiler_out_of_find", "r")
        out = f.read()
        f.close()

        if is_adir_left()
            $out_of_find_ldir = out
        else
            $out_of_find_rdir = out
        end

        reread()
    else
        set_signal_mfiler2()
    end
end

def mlocate(cmd)
    puts "searching disk..."
    
    set_signal_clear()
    if system(cmd)
        set_signal_mfiler2()

        if is_adir_left()
            $locate_path_before_ldir = adir_path()
        else
            $locate_path_before_rdir = adir_path()
        end

        dir_move("/")

        f = open("/tmp/mfiler_out_of_locate", "r")
        out = f.read()
        f.close()

        if is_adir_left()
            $out_of_locate_ldir = out
            $locate_focusback_ldir = view_focusback(0)
            set_view_focusback(0, true)
        else
            $out_of_locate_rdir = out
            $locate_focusback_rdir = view_focusback(1)
            set_view_focusback(1, true)
        end

        toggle_nameonly_mode()

        reread()
    else
        set_signal_mfiler2()
    end
end

$visible_dot_dir = true

def set_glob_mask_hook()
    def mask_hook_ldir(fname, kind)
        if ldir_mask == "{*,.*}"
            return nil
        else
            if fname == ".."
                return true
            elsif !$visible_dot_dir && (kind == 1 || kind == 3) && fname[0] == "."[0]
                return false
            elsif File.fnmatch?(ldir_mask, fname) || kind == 1 || kind == 3
                return true
            else
                return false
            end
        end
    end

    def mask_hook_rdir(fname, kind)
        if rdir_mask == "{*,.*}"
            return nil
        else
            if fname == ".."
                return true
            elsif !$visible_dot_dir && (kind == 1 || kind == 3) && fname[0] == "."[0]
                return false
            elsif File.fnmatch?(rdir_mask, fname) || kind == 1 || kind == 3
                return true
            else
                return false
            end
        end
    end
end

def glob_mask(mask)
    set_glob_mask_hook()
    set_mask(mask)
    set_mask_sdir(mask)
    reread()

    if $sftp_path
        sftp_path_refresh()
    end
end

def set_regex_mask_hook()
    def mask_hook_ldir(fname, kind)
        begin
            if fname =~ /#{ldir_mask()}/ || kind == 1 || kind == 3
                return true
            else
                return false
            end
        rescue
            return false
        end
    end
    def mask_hook_rdir(fname, kind)
        begin
            if fname =~ /#{rdir_mask()}/ || kind == 1 || kind == 3
                return true
            else
                return false
            end
        rescue
            return false
        end
    end
end

def regex_mask(mask) 
    set_regex_mask_hook()
    set_mask(mask)
    set_mask_sdir(mask)
    reread()

    if $sftp_path
        sftp_path_refresh()
    end
end

defmenu("grep_menu",
    "(g)rep",               KEY_g,      "grep_current_directory()",
    "(G)rep recursive",     KEY_G,      "grep_recursive()",
    "(m)grep",              KEY_m,      "mgrep_current_directory()",
    "(M)grep recursive",    KEY_M,      "mgrep_recursive()"
)

defmenu("find",
    "(a)ll",          KEY_a,  "cmdline2(\"%R%q mfind(\\\"/usr/bin/find .  -follow 2> /dev/null | tee /tmp/mfiler_out_of_find\\\")\", 29, nil)",
    "(n)ame",          KEY_n,  "cmdline2(\"%R%q mfind(\\\"/usr/bin/find . -name '' -follow 2> /dev/null | tee /tmp/mfiler_out_of_find\\\")\", 36, nil)",
    "(r)egex",         KEY_r,  "cmdline2(\"%R%q mfind(\\\"/usr/bin/find . -regex '' -follow 2> /dev/null | tee /tmp/mfiler_out_of_find\\\")\", 37, nil)",
    "(s)ize under",          KEY_s,  "cmdline2(\"%R%q mfind(\\\"/usr/bin/find . -size -k -follow 2> /dev/null | tee /tmp/mfiler_out_of_find\\\")\", 36, nil)",
    "(S)ize above",          KEY_S,  "cmdline2(\"%R%q mfind(\\\"/usr/bin/find . -size +k -follow 2> /dev/null | tee /tmp/mfiler_out_of_find\\\")\", 36, nil)",
    "S(i)ze just",          KEY_i,  "cmdline2(\"%R%q mfind(\\\"/usr/bin/find . -size k -follow 2> /dev/null | tee /tmp/mfiler_out_of_find\\\")\", 35, nil)",
    "modified (t)ime within", KEY_t,  "cmdline2(\"%R%q mfind(\\\"/usr/bin/find . -mtime - -follow 2> /dev/null | tee /tmp/mfiler_out_of_find\\\")\", 37, nil)",
    "modified (m)in within",  KEY_m,  "cmdline2(\"%R%q mfind(\\\"/usr/bin/find . -mmin - -follow 2> /dev/null | tee /tmp/mfiler_out_of_find\\\")\", 36, nil)",
    "(u)ser",          KEY_u,  "cmdline2(\"%R%q mfind(\\\"/usr/bin/find . -user  -follow 2> /dev/null | tee /tmp/mfiler_out_of_find\\\")\", 35, nil)",
    "(g)roup",         KEY_g,  "cmdline2(\"%R%q mfind(\\\"/usr/bin/find . -group  -follow 2> /dev/null | tee /tmp/mfiler_out_of_find\\\")\", 36, nil)",
    "(p)ermision",     KEY_p,  "cmdline2(\"%R%q mfind(\\\"/usr/bin/find . -perm  -follow 2> /dev/null | tee /tmp/mfiler_out_of_find\\\")\", 35, nil)",
    "t(y)pe",          KEY_y,  "cmdline2(\"%R%q mfind(\\\"/usr/bin/find . -type  -follow 2> /dev/null | tee /tmp/mfiler_out_of_find\\\")\", 35, nil)",
    "gr(e)p",          KEY_e,  "cmdline2(\"%R%q mvgrep(\\\"grep '' * 2> /dev/null | tee /tmp/mfiler_out_of_grep2\\\")\", 20, nil)",
    "gr(E)p -r",          KEY_E,  "cmdline2(\"%R%q mvgrep(\\\"grep -r '' * 2> /dev/null | tee /tmp/mfiler_out_of_grep2\\\")\", 23, nil)",
    "(l)ocate",          KEY_l,  "cmdline2(\"%R%q mlocate(\\\"locate  2> /dev/null | tee /tmp/mfiler_out_of_locate\\\")\", 22, nil)"
)

$list_mode = false

def toggle_list_mode()
    if !$list_mode 
        $list_mode = !$list_mode

        $list_mode_filer = view_filer()
        $list_mode_mtime = view_mtime()
        $list_mode_size = view_size()
        $list_mode_permission = view_permission()
        $list_mode_user = view_user()
        $list_mode_group = view_group()
        $list_mode_nlink = view_nlink()

        set_view_filer("one_dir")
        set_view_mtime(true)
        set_view_size(true)
        set_view_permission(true)
        set_view_user(true)
        set_view_group(true)
        set_view_nlink(true)
    else
        $list_mode = !$list_mode

        set_view_filer($list_mode_filer)
        set_view_mtime($list_mode_mtime)
        set_view_size($list_mode_size)
        set_view_permission($list_mode_permission)
        set_view_user($list_mode_user)
        set_view_group($list_mode_group)
        set_view_nlink($list_mode_nlink)
    end
end

$nameonly_mode = false

def toggle_nameonly_mode()
    if !$nameonly_mode 
        $nameonly_mode = !$nameonly_mode

        $nameonly_mode_filer = view_filer()
        $nameonly_mode_mtime = view_mtime()
        $nameonly_mode_size = view_size()
        $nameonly_mode_permission = view_permission()
        $nameonly_mode_user = view_user()
        $nameonly_mode_group = view_group()
        $nameonly_mode_nlink = view_nlink()
        $nameonly_mode_fname_divide_ext = view_fname_divide_ext()

        set_view_filer("one_dir")
        set_view_mtime(false)
        set_view_size(false)
        set_view_permission(false)
        set_view_user(false)
        set_view_group(false)
        set_view_nlink(false)
        set_view_fname_divide_ext(false)
    else
        $nameonly_mode = !$nameonly_mode

        set_view_filer($nameonly_mode_filer)
        set_view_mtime($nameonly_mode_mtime)
        set_view_size($nameonly_mode_size)
        set_view_permission($nameonly_mode_permission)
        set_view_user($nameonly_mode_user)
        set_view_group($nameonly_mode_group)
        set_view_nlink($nameonly_mode_nlink)
        set_view_fname_divide_ext($nameonly_mode_fname_divide_ext)
    end
end

defmenu("view",
   "2dir1 (a)ll", KEY_a, "set_view_filer('all')",
   "1dir(1)", KEY_1, "set_view_filer('one_dir')",
   "1dir(2)", KEY_2, "set_view_filer('one_dir2')",
   "1dir(3)", KEY_3, "set_view_filer('one_dir3')",
   "1dir(5)", KEY_5, "set_view_filer('one_dir5')",
   "toggle (c)olor", KEY_c, "set_view_color(!view_color())",
   "toggle m(t)ime", KEY_t, "set_view_mtime(!view_mtime())",
   "toggle (s)ize", KEY_s, "set_view_size(!view_size())",
   "toggle (p)ermission", KEY_p, "set_view_permission(!view_permission())",
   "toggle (u)ser", KEY_u, "set_view_user(!view_user())",
   "toggle (g)roup", KEY_g, "set_view_group(!view_group())",
   "toggle nl(i)nk", KEY_i, "set_view_nlink(!view_nlink())",
   "toggle fname (d)ivide ext", KEY_d, "set_view_fname_divide_ext(!view_fname_divide_ext())",
   "toggle (f)ocusback", KEY_f, "set_view_focusback(adir_num(), !view_focusback(adir_num()))",
   "toggle (r)emovedir", KEY_r, "set_view_removedir(adir_num(), !view_removedir(adir_num()))",
   "glob (m)ask", KEY_m, "if is_remain_marks() then cmdline2(\"%Q%r glob_mask('')\", -2, nil); else cmdline2(\"%Q%r%M glob_mask('')\", -2, nil); end",
   "regex (M)ask", KEY_M, "if is_remain_marks() then cmdline2(\"%Q%r regex_mask('')\", -2, nil); else cmdline2(\"%Q%r%M regex_mask('')\", -2, nil); end",
   "toggle (l)ist mode", KEY_l, "toggle_list_mode",
   "toggle (n)ameonly mode", KEY_n, "toggle_nameonly_mode",
   "filename (k)anjicode", KEY_k, "menu('filename_kanjicode')"
)

def menu_view()
    menu("view")
end

defmenu("filename_kanjicode",
   "filename kanjicode = '(u)tf'", KEY_u, "set_kanjicode_filename('utf8'); reread()",
   "filename kanjicode = 'utf-(m)ac'", KEY_m, "set_kanjicode_filename('utf8-mac'); reread()",
   "filename kanjicode = '(e)ucj'", KEY_e, "set_kanjicode_filename('eucjp'); reread()",
   "filename kanjicode = '(s)jis'", KEY_s, "set_kanjicode_filename('sjis'); reread()",
   "filename kanjicode = 'unknown' (a)uto", KEY_a, "set_kanjicode_filename('unknown'); reread()"
)

defmenu("sort",
   "(n)ame", KEY_n, "set_sort_kind(adir_num(), 'name')",
   "(N)ame reverse", KEY_N, "set_sort_kind(adir_num(), 'name_rev')",
   "(e)xtension", KEY_e, "set_sort_kind(adir_num(), 'extension')",
   "(E)xtension reverse", KEY_E, "set_sort_kind(adir_num(), 'extension_rev')",
   "(s)ize", KEY_s, "set_sort_kind(adir_num(), 'size')",
   "(S)ize reverse", KEY_S, "set_sort_kind(adir_num(), 'size_rev')",
   "(t)ime", KEY_t, "set_sort_kind(adir_num(), 'time')",
   "(T)ime reverse", KEY_T, "set_sort_kind(adir_num(), 'time_rev')",
   "(u)ser", KEY_u, "set_sort_kind(adir_num(), 'user')",
   "(U)ser reverse", KEY_U, "set_sort_kind(adir_num(), 'user_rev')",
   "(g)roup", KEY_g, "set_sort_kind(adir_num(), 'group')",
   "(G)roup reverse", KEY_G, "set_sort_kind(adir_num(), 'group_rev')",
   "(p)ermission", KEY_p, "set_sort_kind(adir_num(), 'permission')",
   "(P)ermission reverse", KEY_P, "set_sort_kind(adir_num(), 'permission_rev')",
   "none (0)", KEY_0, "set_sort_kind(adir_num(), 'none')",
   "toggle (d)ir up", KEY_d, "set_sort_dir_up(adir_num(), !sort_dir_up(adir_num()))"
   );

if `uname -s`.index("CYGWIN")
    defmenu("attribute",
       "ch(m)od (permission)", KEY_m, "cmdline2('chmod  %m1', 7, nil)",
       "ch(o)wn (owner)", KEY_o, "cmdline2('chown  %m1', 7, nil)",
       "ch(g)rp (group)", KEY_g, "cmdline2('chgrp  %m1', 7, nil)",
       "ch(M)od -R (permission recursive)", KEY_M, "cmdline2('chmod -R  %m1', 10, nil)",
       "ch(O)wn -R (owner recursive)", KEY_O, "cmdline2('chown -R  %m1', 10, nil)",
       "ch(G)rp -R (group recursive)", KEY_G, "cmdline2('chgrp -R  %m1', 10, nil)",
       "m(a)ttr (permission and date)", KEY_a, "shell('mattr %m1', 'mattr')",
       "m(A)ttr (windows attribute)", KEY_A, "shell('mattrcyg %m1', 'mattrcyg')");
else
    defmenu("attribute",
       "ch(m)od (permission)", KEY_m, "cmdline2('chmod  %m1', 7, nil)",
       "ch(o)wn (owner)", KEY_o, "cmdline2('chown  %m1', 7, nil)",
       "ch(g)rp (group)", KEY_g, "cmdline2('chgrp  %m1', 7, nil)",
       "ch(M)od -R (permission recursive)", KEY_M, "cmdline2('chmod -R  %m1', 10, nil)",
       "ch(O)wn -R (owner recursive)", KEY_O, "cmdline2('chown -R  %m1', 10, nil)",
       "ch(G)rp -R (group recursive)", KEY_G, "cmdline2('chgrp -R  %m1', 10, nil)",
       "m(a)ttr (permission and date)", KEY_a, "shell('mattr %m1', 'mattr')");
end

defmenu("file_attr_sudo",
   "sudo ch(m)od (permission)", KEY_m, "cmdline2('sudo chmod  %m1', 12, nil)",
   "sudo ch(o)wn (owner)", KEY_o, "cmdline2('sudo chown  %m1', 12, nil)",
   "sudo ch(g)rp (group)", KEY_g, "cmdline2('sudo chgrp  %m1', 12, nil)",
   "sudo ch(M)od -R (permission recursive)", KEY_M, "cmdline2('sudo chmod -R  %m1', 15, nil)",
   "sudo ch(O)wn -R (owner recursive)", KEY_O, "cmdline2('sudo chown -R  %m1', 15, nil)",
   "sudo ch(G)rp -R (group recursive)", KEY_G, "cmdline2('sudo chgrp -R  %m1', 15, nil)",
   "sudo m(a)ttr (permission and date)", KEY_a, "shell('sudo mattr %m1', 'mattr')");

def key_sudo_unpack()
    ext = cursor_ext()
    if ext == "gz" 
        if cursor_name().rindex(".tar.gz")
            if is_one_dir()
                cmdline2('sudo tar xvfz %f -C %D1', -1, nil)
            else
                cmdline2('sudo tar xvfz %f -C %D2', -1, nil)
            end
        else
            cmdline2('sudo gzip -d %F', -1, nil)
        end
    elsif ext == "bz2"
        if cursor_name().rindex(".tar.bz2")
            if is_one_dir()
                cmdline2('sudo tar xvfj %f -C %D1', -1, nil)
            else
                cmdline2('sudo tar xvfj %f -C %D2', -1, nil)
            end
        else
            cmdline2('sudo bzip2 -d %f', -1, nil)
        end
    elsif ext == "tar"
        if is_one_dir() 
            cmdline2('sudo tar xvf %f -C %D1', -1, nil) 
        else 
            cmdline2('sudo tar xvf %f -C %D2', -1, nil)
        end
    elsif ext == "tgz"
        if is_one_dir()
            cmdline2('sudo tar xvfz %f -C %D1', -1, nil)
        else
            cmdline2('sudo tar xvfz %f -C %D2', -1, nil)
        end
    elsif ext == "lzh"
        if is_one_dir()
            cmdline2('%S cd %D1; sudo lha x %F', adir_path().size() + 8, nil)
        else 
            cmdline2('%S cd %D2; sudo lha x %F', sdir_path().size() + 8, nil)
        end
    elsif ext == "zip" || ext == "ZIP"
        if is_one_dir()
            cmdline2('sudo unzip %f -d %D1', -1, nil)
        else
            cmdline2('sudo unzip %f -d %D2', -1, nil)
        end
    end
end

def key_sudo_Unpack()
    ext = cursor_ext()
    
    if ext == "gz"
        if cursor_name().rindex(".tar.gz")
            cmdline2('sudo tar xvfz %f -C %D1', -1, nil)
        else
            cmdline2('sudo gzip -d %F', -1, nil)
        end
    elsif ext == "bz2"
        if cursor_name().rindex(".tar.bz2")
            cmdline2('sudo tar xvfj %f -C %D1', -1, nil)
        else
            cmdline2('sudo bzip2 -d %f', -1, nil)
        end
    elsif ext == "tar"
        cmdline2('sudo tar xvf %f -C %D1', -1, nil)
    elsif ext == "tgz"
        cmdline2('sudo tar xvfz %f -C %D1', -1, nil)
    elsif ext == "zip" || ext == "ZIP"
        cmdline2('sudo unzip %f -d %D1', -1, nil)
    elsif ext == "lzh"
        cmdline2('%S cd %D1; sudo lha x %F', adir_path().size() + 8, nil)
    end
end

def key_sudo_excute()
    if File.stat(adir_path() + cursor_name()).executable? && !File.directory?(adir_path() + cursor_name())
        cmdline2('sudo ./%f ', 0, nil)
    elsif cursor_name().rindex(".pl")
        cmdline2('sudo perl %f ', 0, nil)
    elsif cursor_name().rindex(".py")
        cmdline2('sudo python %f ', 0, nil)
    elsif cursor_name().rindex(".rb")
        cmdline2('sudo ruby %f ', 0, nil)
    else
        cmdline2('sudo  ./%f', 6, nil)
    end
end

def key_sudo_backup()
    cmdline2("sudo cp #{CP_OPTION} %f '" + cursor_name() + '.bak\'', -1, nil)
end

defmenu("sudo-menu",
   "sudo (c)opy", KEY_c,   "if is_one_dir() then cmdline2('sudo cp #{CP_OPTION} %m1 %D1', -1, nil) else cmdline2('sudo cp #{CP_OPTION} %m1 %D2', -1, nil) end",
   "sudo (m)v", KEY_m,      "if is_one_dir() then cmdline2('sudo mv %m1 %D1', -1, nil) else cmdline2('sudo mv %m1 %D2', -1, nil) end",
   "sudo (d)elete",  KEY_d, "cmdline2('sudo rm -R %m1', 0, nil)",
   "sudo (r)ename", KEY_r, "cmdline2('sudo mv %f %f', -1, nil)",
   "sudo (a)ttr menu",  KEY_a, "menu('file_attr_sudo')",
   "sudo (e)dit", KEY_e,   "shell('%q sudo $EDITOR %f', cursor_name_convert)",
   "sudo (v)iewer", KEY_v,     "shell('%q sudo $PAGER %f', cursor_name_convert)",
   "sudo (p)ack", KEY_p, "if is_one_dir() then cmdline2('sudo tar cvfz %X.tgz %m1', -9, nil) else cmdline2('sudo tar cvfz %D2%X.tgz %m1', -9, nil) end", 
   "sudo (P)ack", KEY_P,     "cmdline2('sudo tar cvfz %X.tgz %m1', -9, nil)",
   "sudo (u)npack", KEY_u,     "key_sudo_unpack()",
   "sudo (U)npack", KEY_U,     "key_sudo_Unpack()",
   "sudo ma(k)e dir", KEY_k,   "cmdline_noconvert(\"sudo mkdir -p ''\", -1, nil)", 
   "sudo (n)ew file", KEY_n,   "cmdline_noconvert(\"sudo touch ''\", -1, nil)",
   "sudo make (l)ink", KEY_l,  "if is_one_dir() then cmdline2('sudo ln -s %M1 %D1', -1, nil) else cmdline2('sudo ln -s %M1 %D2', -1, nil) end",
   "sudo make (L)ink", KEY_L,  "cmdline2('sudo ln -s %F %D1', -1, nil)",
   "sudo (g)rep", KEY_g,       "cmdline2(\"%q sudo grep '' * -n 2> /dev/null | $PAGER\", 15, nil)",
   "sudo (G)rep", KEY_G,       "cmdline2(\"%q sudo grep -R '' * -n 2> /dev/null | $PAGER\", 17, nil)",
   "sudo e(x)ecute", KEY_x, "key_sudo_excute()",
   "sudo (b)ackup", KEY_b, "key_sudo_backup()",
   "sudo run shell(:)", KEY_COLON, "shell('%q echo \"-+- \"%D1\" -+-\"; ls -aFG; sudo $SHELL', \"#{ENV['SHELL']}\")", 
   "sudo s(h)ell", KEY_h,   "cmdline2('sudo ', 0, nil)"
   );

def select_job()
    cmdline2("%r%Q forground_job ", 0, nil);
end

def add_current_path_to_directory_menu()
    cmdline2("%r%Q add_directory_menu \"#{adir_path}\", KEY_1", 26, nil)
end

defmenu("extra-menu",
   "(1) back to previous direcotry",                KEY_1, "dir_back()",
   "(2) add cursor file to making playlist",        KEY_2, "add_cursor_file_to_making_playlist()",
   "(3) set cursor file as making playlist file",   KEY_3, "change_making_playlist()",
   "(c) copy",   KEY_c, "copy2()",
   "(x) cut",    KEY_x, "cut()",
   "(v) past",   KEY_v, "past()"
   );

defmenu("glob_mark_menu",
    "(f)ile", KEY_f,      "cmdline2(\"%r%Q%M glob_mark('*')\", -2, nil)",
    "(d)irectory", KEY_d, "cmdline2(\"%r%Q%M glob_mark_dir('*')\", -2, nil)"
    );

defmenu("regex_mark_menu",
    "(f)ile", KEY_f, "cmdline2(\"%r%Q%M regex_mark(//)\", -2, nil)",
    "(d)irectory", KEY_d, "cmdline2(\"%r%Q%M regex_mark_dir(//)\", -2, nil)"
    );

defmenu("mark_menu",
    "(1) glob mark file",                      KEY_1, "cmdline2(\"%r%Q%M glob_mark '*'\", -1, nil)",
    "(2) glob mark directory",                 KEY_2, "cmdline2(\"%r%Q%M glob_mark_dir '*'\", -1, nil)",
    "(3) glob mark all",                       KEY_3, "cmdline2(\"%r%Q%M glob_mark_all '*'\", -1, nil)",
    "(4) regex mark file",                     KEY_4, "cmdline2(\"%r%Q%M regex_mark //\", -1, nil)",
    "(5) regex mark directory",                KEY_5, "cmdline2(\"%r%Q%M regex_mark_dir //\", -1, nil)",
    "(6) regex mark all",                      KEY_6, "cmdline2(\"%r%Q%M regex_mark_all //\", -1, nil)",
    "(7) mark_clear",                          KEY_7, "mark_clear()",
    "(8) mark_all",                            KEY_8, "mark_all()",
    "(9) mark_all_files",                      KEY_9, "mark_all_files()",
    "(a) mark_directory",                      KEY_a, "mark_directory()",
    "(b) mark_executable",                     KEY_b, "mark_executable()",
    "(c) mark_symlink",                        KEY_c, "mark_symlink()",
    "(d) mark_user file",                      KEY_d, "cmdline2('%r%Q%M mark_user \"\"', -1, nil)",
    "(e) mark_user directory",                 KEY_e, "cmdline2('%r%Q%M mark_user_dir \"\"', -1, nil)",
    "(f) mark_user all",                       KEY_f, "cmdline2('%r%Q%M mark_user_all \"\"', -1, nil)",
    "(g) mark_group file",                     KEY_g, "cmdline2('%r%Q%M mark_group \"\"', -1, nil)",
    "(h) mark_group directory",                KEY_h, "cmdline2('%r%Q%M mark_group_dir \"\"', -1, nil)",
    "(i) mark_group all",                      KEY_i, "cmdline2('%r%Q%M mark_group_all \"\"', -1, nil)",
    "(j) mark_with_comparison",                KEY_j, "mark_with_comparison()"
    );

def goto_cmdline()
    cmdline2('', 0, nil)
end

def goto_cmdline_extern_shell()
    cmdline2('%S ', 0, nil)
end

def excute_mark_file()
    cmdline2('%S for i in %m1; do  ./$i; done', 21, nil)
end

def excute_cursor_or_mark_file()
    if marking
        excute_mark_file()
    else
        excute_cursor_file()
    end
end

def load_cursor_file()
    cmdline2('%r%Q load "' + cursor_name_convert + '"', 0, nil)
end

def call_shell()
    shell('%q echo "-+- "%D1" -+-"; ls -aFG; $SHELL', "#{ENV['SHELL']}")
end

$default_one_dir = "one_dir2"

def quit_or_close_directory()
    if $sftp_path
        sftp_end()
    else 
        if is_one_dir()
            mf_exit()
        else
            if is_adir_left
                set_view_filer($default_one_dir)
            else
                cursor_other()
                set_view_filer($default_one_dir)
            end
        end
    end
end

def quit()
    if $sftp_path
        sftp_end()
    else 
        mf_exit()
    end
end

def toggle_1directory_mode_and_2directory_mode()
    if is_one_dir()
        $default_one_dir = view_filer
        
        if is_adir_left
            sdir_move(adir_path())
            set_view_filer("all")
            cursor_other()
            set_scrolltop(0)
            cursor_move(1)
        else
            cursor_other()
            dir_move(sdir_path())
            set_view_filer("all")
            cursor_other()
            set_scrolltop(0)
            cursor_move(1)
        end
    else
        if $sftp_path
            sftp_end()
        else
            if is_adir_left
                set_view_filer($default_one_dir)
            else
                cursor_other()
                set_view_filer($default_one_dir)
            end
        end
    end
end

def goto_2directory_mode()
    if is_one_dir()
        if $option_tab_remain_dir
            $default_one_dir = view_filer

            set_view_filer("all")
            cursor_other()
        else
            $default_one_dir = view_filer
            
            if is_adir_left
                sdir_move(adir_path())
                set_view_filer("all")
                cursor_other()
                set_scrolltop(0)
                cursor_move(1)
            else
                cursor_other()
                dir_move(sdir_path())
                set_view_filer("all")
                cursor_other()
                set_scrolltop(0)
                cursor_move(1)
            end
        end
    else
        cursor_other()
    end
end

def stack_current_directory()
    tab_new(tab_max(), adir_path(), scrolltop(), cursor())
    dir_move('~')
end

def close_current_directory()
    if tab_max() == 0
        dir_move('~')
    else
        tab_up(tab_max()-1)
        tab_close(tab_max()-1)
    end
end

def open_tab(num)
    tab_new(tab_max(), adir_path(), scrolltop(), cursor())
    tab_up(num)
    tab_close(num)
end

def unpack_cursor_file()
    if cursor_name().index(".gz")
        unpack_gz()
    elsif cursor_name().index(".bz2")
        unpack_bz2()
    elsif cursor_name().index(".tar")
        unpack_tar()
    elsif cursor_name().index(".tgz")
        unpack_tgz()
    elsif cursor_name().index(".lzh")
        unpack_lzh()
    elsif cursor_name().index(".zip")
        unpack_zip()
    end
end

def unpack_cursor_file2()
    if cursor_name().index(".gz")
        unpack_gz2()
    elsif cursor_name().index(".bz2")
        unpack_bz22()
    elsif cursor_name().index(".tar")
        unpack_tar2()
    elsif cursor_name().index(".tgz")
        unpack_tgz2()
    elsif cursor_name().index(".lzh")
        unpack_lzh2()
    elsif cursor_name().index(".zip")
        unpack_zip2()
    end
end

def msed()
    if marking()
        sed_mark_file()
    else
        sed_current_directory()
    end
end

def minfo()
    if File.directory?(adir_path() + cursor_name())
        info_directory()
    else
        info_file()
    end
end

defmenu("all",
    "(c)opy", KEY_c, "copy_cursor_or_mark_file()",
    "(C)opy", KEY_C, "menu_copy()",
    "(m)ove", KEY_m, "move_mark_or_cursor_file()",
    "(M)ove", KEY_M,  "menu_move()",
    "(d) trashbox", KEY_d, "throw_mark_or_cursor_file()",
    "(D)elete", KEY_D, "menu_delete()",
    "(e)dit", KEY_e, "edit_cursor_file()",
    "(E)dit", KEY_E, "edit_new_file()",
    "(v)iew", KEY_v, "view_cursor_file()",
    "(r)ename", KEY_r, "rename_cursor_or_mark_file()",
    "(R)ename", KEY_R, "rename_cursor_file2()",
    "(b)ackup", KEY_b, "backup_cursor_or_mark_file()",
    "(p)ack", KEY_p, "pack_mark_or_cursor_file()",
    "(P)ack", KEY_P, "pack_mark_or_cursor_file2()",
    "ma(k)e directory", KEY_k, "make_directory()",
    "(n)ew file", KEY_n, "make_new_file_cmdline()",
    "(l)ink", KEY_l, "make_link()",
    "(L)ink", KEY_L, "make_link2()",
    "(u)npack", KEY_u, "unpack_cursor_file()",
    "(U)npack", KEY_U, "unpack_cursor_file2()",
    "(f) isearch", KEY_f, "start_isearch()",
    "(F) isearch partmatch", KEY_F, "start_isearch_partmatch()",
    "e(x)cute", KEY_x, "excute_cursor_or_mark_file()",
    "(h) cmdline", KEY_h, "goto_cmdline()",
    "(H) cmdline menu", KEY_H, "menu_shell()",
    "mma(K)e", KEY_K, "mmake ENV['EDITOR'], '+'",
    "m(g)rep", KEY_g, "mgrep_current_directory()",
    "(G)rep", KEY_G, "menu_grep()",
    "(S)ed", KEY_S, "msed",
    "(i)nfo", KEY_i, "minfo",
    "(a)ttribute", KEY_a, "menu_attr()",
    "(s)ort", KEY_s, "menu_sort()",
    "(V)iew menu", KEY_V, "menu_view()",
    "m(y) commands", KEY_y, "menu_my_commands()",
    "(j)ump", KEY_j, "directory_menu",
    "(J)ump", KEY_J, "menu_jump()",
    "c(o)py directory path", KEY_o, "copy_path()",
    "c(O)py directory path", KEY_O, "copy_path2()",
    "(w) exchange directory path", KEY_w, "exchange_path()",
    "(z) xterm", KEY_z, "toggle_xterm_next()",
    "(Z) xterm2", KEY_Z, "toggle_xterm()",
    "(q)uite", KEY_q, "quit_or_close_directory()",
    "(Q)uite", KEY_Q, "mf_exit()"
    )

def enter_directory()
    if $sftp_path && is_adir_right
        if cursor_name() == ".."
            move_parent_dir()
        else
            sftp_change_dir($sftp_path + cursor_name() + "/")
        end
    else
        dir_move(cursor_name())
    end
end

def enter_link()
    if File.directory?(cursor_name)
        dir_move(cursor_name())
    end
end

def enter_mark()
    cmdline2(' %m1', 1, nil)
end

def enter_gz()
    if cursor_name().rindex(".tar.gz")
        cmdline2('%q tar tvfz %f | $PAGER', 0, nil)
    else
        cmdline2('%q zcat %f | $PAGER', 0, nil)
    end

    enter_cursor_down()
end

def enter_tgz()
    cmdline2('%q tar tvfz %f | $PAGER', 0, nil)

    enter_cursor_down()
end

def etner_bz2()
    if cursor_name().rindex(".tar.bz2")
        cmdline2('%q tar tvfj %f | $PAGER', 0, nil)
    else
        cmdline2('%q bzcat %f | $PAGER', 0, nil)
    end

    enter_cursor_down()
end

def enter_lzh()
    cmdline2('%q lha l %f | $PAGER', 0, nil)
    enter_cursor_down()
end

def enter_zip()
    cmdline2('%q zipinfo %f | $PAGER', 0, nil)
    enter_cursor_down()
end

def edit_and_load_ruby_script()
    shell("%q%s%M $EDITOR /tmp/mfiler_ruby_script.rb", "/tmp/mfiler_ruby_script.rb")
    s = select_menu_mfiler("Would you like to excute /tmp/mfiler_ruby_script.rb?", ["Yes", "No"])
    if s == 0
        if File.exist?("/tmp/mfiler_ruby_script.rb")
            shell("%R%M load '/tmp/mfiler_ruby_script.rb'", "ruby_script")
        end
        reread()
    end
end

# ファイルをマスクして補完
def completion_mask(editing_dir, editing_file, editing, add_space, reg)

#message("editing (%s) editing_dir (%s) editing_file (%s)", editing, editing_dir, editing_file)

    reg = Regexp.new(reg)

    if editing_dir != ""
        if editing_dir[0] == "$"[0]
            i = 1
            env_name = ""
    
            while editing_dir[i] != "/"[0]
                env_name = env_name +  editing_dir[i].chr
                i = i + 1
            end
            
            editing_dir = ENV[env_name] + editing_dir[i..editing_dir.size]
        elsif editing_dir[0] == "~"[0]
            editing_dir = ENV['HOME'] + editing_dir[1..editing_dir.size]
        end
    
        begin
            array = []
    
            Dir.foreach(editing_dir) { |file|
                if file != "." && file != ".."
                    if File.exist?(editing_dir + file) && File.stat(editing_dir + file).directory?
                        file = file + "/"
                    end
    
                    if reg.match(file)
                        #file = editing_dir + file
                        array.push(file)
                    end
                end
            }
    
            array.sort!
            completion(array, editing_file, add_space)
            
        rescue SystemCallError
            message("SystemCallError")        
        end
    else
        if editing[0] == '$'[0]
            array = []
            ENV.each {|key, value|
                if File.exist?(value) && File.stat(value).directory? 
                    array.push("$" + key + "/")
                else
                    array.push("$" + key)
                end
            }

            completion(array, editing, add_space)
        elsif editing[0] == '~'[0]
            array = ["~/"]

            completion(array, editing, add_space)

        else
            begin
               array = [] 
               Dir.foreach(adir_path()) {|file|
                   path = adir_path() + file
                   if File.exist?(path) && File.stat(path).directory?
                      file = file + "/"
                   end

                   if file != "./" && file != "../" && reg.match(file)
                       array.push(file)
                   end
                }

                array.sort!
                completion(array, editing, add_space)

            rescue SystemCallError
                message("SystemCallError")
            end
        end
    end

    return nil
end

###############################################################################
# ** 追加プログラム ***
###############################################################################

###############################################################################
# 新規ファイル作成メニュー(N)
###############################################################################
def make_new_shell_script(fname)
    if !File.exist?(adir_path() + fname)
        begin 
            f = File.new(adir_path() + fname,File::CREAT|File::TRUNC|File::WRONLY)
            f.chmod(0755)
            f.print("#!/bin/sh\n\n")
            f.close
        rescue
            return false
        end
        reread()
        cursor_move(file_num(fname))
    end
end

def make_new_html(fname)
    if !File.exist?(adir_path() + fname)
        begin 
            f = File.new(adir_path() + fname,File::CREAT|File::TRUNC|File::WRONLY)
            f.print <<EOS
<html>
<head>
<title></title>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8">
</head>
<body>
</body>
</html>
EOS
            f.close
        rescue
            return false
        end
        reread()
        cursor_move(file_num(fname))
    end
end

def make_new_perl_script(fname)
    if !File.exist?(adir_path() + fname)
        begin
            f = File.new(adir_path() + fname,File::CREAT|File::TRUNC|File::WRONLY)
            f.chmod(0755)
            f.print("#!/usr/bin/env perl\n\n")
            f.close
        rescue
            return false
        end
        reread()
        cursor_move(file_num(fname))
    end
end

def make_new_python_script(fname)
    if !File.exist?(adir_path() + fname)
        begin
            f = File.new(adir_path() + fname,File::CREAT|File::TRUNC|File::WRONLY)
            f.chmod(0755)
            f.print("#!/usr/bin/env python\n\n")
            f.close
        rescue
            return false
        end
        reread()
        cursor_move(file_num(fname))
    end
end

def make_new_ruby_script(fname)
    if !File.exist?(adir_path() + fname)
        begin
            f = File.new(adir_path() + fname,File::CREAT|File::TRUNC|File::WRONLY)
            f.chmod(0755)
            f.print("#!/usr/bin/env ruby\n\n")
            f.close
        rescue
            return false
        end
        reread()
        cursor_move(file_num(fname))
    end
end

defmenu("new-file",
    "(s)h script", KEY_s, "cmdline2('%r%Q make_new_shell_script \\'.sh\\'', -4, nil)",
    "(h)tml", KEY_h, "cmdline2('%r%Q make_new_html \\'.html\\'', -6, nil)",
    "(p)erl script", KEY_p, "cmdline2('%r%Q make_new_perl_script \\'.pl\\'', -4, nil)",
    "p(y)thon script", KEY_y, "cmdline2('%r%Q make_new_python_script \\'.py\\'', -4, nil)",
    "(r)uby script", KEY_r, "cmdline2('%r%Q make_new_ruby_script \\'.rb\\'', -4, nil)",
    "play (l)ist", KEY_l, "cmdline2('%r%Q make_new_playlist \\'.list\\'', -6, nil)"
);

keycommand NOMETA, KEY_N,      "*",     "menu('new-file')"

###############################################################################
# ディレクトリメニュー
###############################################################################
$directory_menu = [
    ["(1) " + ENV['HOME'] + "/tmp/", KEY_1, ENV['HOME'] + "/tmp/"], 
    ["(2) " + "/etc", KEY_2, "/etc/"]
]

def directory_menu()
    menu_array = []
    $directory_menu.each() {|item|
        menu_array.push(item[0])
        menu_array.push(item[1])
        menu_array.push("dir_move '#{item[2]}'")
    }

    menu_array.push("----------------------------------------")
    menu_array.push(KEY_CTRL_Z)
    menu_array.push("")

    menu_array.push("(a)dd currenct path to jump menu")
    menu_array.push(KEY_a)
    menu_array.push("add_current_path_to_directory_menu()")

    menu_array.push("(d)elete jump menu")
    menu_array.push(KEY_d)
    menu_array.push("del_directory_menu()")

    menu_array.push("(c)lear jump menu")
    menu_array.push(KEY_5)
    menu_array.push("clear_directory_menu()")

    defmenu('jump', *menu_array )
    menu('jump')
end

def add_directory_menu(title, key)
    $directory_menu.push([title, key, adir_path()])
end

def del_directory_menu()
    menu_array = []
    n = 0
    $directory_menu.each() {|item|
        menu_array.push(item[0])
        menu_array.push(item[1])
        menu_array.push("$directory_menu.delete_at #{n}")
        n += 1
    }
    defmenu( 'delete_directory_menu', *menu_array )
    menu('delete_directory_menu')
end

def clear_directory_menu()
    $directory_menu = []
end

def save_directory_menu(f)
    f.puts $directory_menu.size

    $directory_menu.each() {|item|
        f.puts item[0]
        f.puts item[1]
        f.puts item[2]
    }
end

def read_directory_menu(f)
    $directory_menu = []

    size = f.readline.chomp!.to_i
    
    i = 0
    while i < size
        array = []
        line = f.readline.chomp!
        array.push(line)
        line = f.readline.chomp!
        array.push(line.to_i)
        line = f.readline.chomp!
        array.push(line)

        $directory_menu.push(array)

        i += 1
    end
end


###############################################################################
# ディレクトリヒストリ by Lei
###############################################################################

# ヒストリ数
$my_dir_history_max = 999

$my_dir_history = Array.new()

# ディレクトリ移動メソッドオーバーライド
def dir_move( dir )
    super(dir)

    # dir_move がフルパスでも返してくれれば良かったが無いのでアクティブウィンドウを残す
    $my_dir_history.insert( 0, adir_path )

    # 最大履歴を越えていたら末尾を削除
    if $my_dir_history.size > $my_dir_history_max
        $my_dir_history.pop
    end
end

# 履歴メニュー表示
def menu_dir_history()
    menu_array = Array.new
    counter = 0
    width_max = Integer( mgetmaxx * 4/5 ) # 画面の2/3くらいは表示を許しましょうか
    
    # 引数生成
    $my_dir_history.each do |i|
        menu_array.push "(#{counter+1}) #{my_get_deform_path(i,width_max)}"
        menu_array.push KEY_a
        menu_array.push "dir_move('#{i}')"
        counter += 1 
    end

    # defmenu 宣言 
    if menu_array.size == 0
        defmenu( 'dir_history_menu', "(1) ~ (not history)", KEY_1, "dir_move('~')");
    else
        #message( menu_array.join "," )
        defmenu( 'dir_history_menu', *menu_array )
    end
    # menu 呼び出し
    menu('dir_history_menu')
end

# 指定のpath文字列が指定の長さ以上あれば先頭を.../にして指定文字列以下になるようにして返す
# ただし、最下層ディレクトリの名称がmax_length以上であった場合はmax_length以上の文字列が返される
# （※ディレクトリ名の中に/が入っていたら誤動作するけど入れられましたっけ？）
def my_get_deform_path( path, max_length )
    # 長くなければそのまま返す
    if path.length < max_length
        return path
    end

    # /で区切ってひっくり返して後ろから処理していきます
    dirs = path.split(/\//).reverse()
    path_str = '/' + dirs.shift # 長いディレクトリ名対策に最低1個は表示しよう
    dirs.each do |dir|
    # 長さオーバーチェック。+4は頭に'.../'がつくため
        if path_str.length + dir.length + 4 > max_length
            break
        end
        path_str = '/' + dir + path_str
    end

    path_str = '...' + path_str

    return path_str
end

###############################################################################
# makeのラッパー
###############################################################################
def mmake(editor, line_option)
    if !mis_curses
        puts "curses is not running..."
        return
    end

    maxx = mgetmaxx()

    lines = []
    cursor = 0
    scroll_top = 0

    ### メイクを実行 ###
    mclear()
    mmove(0,0)
    #mmvprintw(0,0, "compiling...")
    mrefresh()

    mendwin()
    set_signal_clear()
    system("make 2>&1 | tee /tmp/mfiler_out_of_make")
    set_signal_mfiler2()
    minitscr()

    f = open("/tmp/mfiler_out_of_make", "r")
    output = f.read()
    f.close()

    if output == ""
        mprintw("no output")
        mrefresh()
        mgetch()
        return
    end

    ### メイクの出力を行ごとにlinesに格納 ###
    i = 0
    tmp = ""
    while i < output.size()
        if output[i] == ?\n
            if tmp != ""
                lines.push(tmp)
            end
            tmp = ""
        else
            tmp = tmp + output[i].chr
        end

        i = i + 1
    end

    ### 画面サイズに併せてぶつ切り ###
    lines2 = []
    lines.each do |line|
        c = 0
        while c < line.size
            line2 = wstrcut(line[c..-1], maxx)
            c += line2.size
            lines2.push line2
        end
    end

    ### メインループ ###
    if lines2.size > 0 
        key = [0, 0]    
        while true
            ### 描写 ###
            mclear()
            n = scroll_top
            while n < lines2.size
                if cursor == n
                    mattron(MA_REVERSE)
                end
                
                mmvprintw(n-scroll_top, 0, "%s", lines2[n])
                
                if cursor == n
                    mattroff()
                end

                n += 1
            end
            
            mrefresh()

            ### インプット ###
            key = mgetch()

            if key[1] == 14 || key[1] == KEY_DOWN
                cursor = cursor + 1
            end
            if key[1] == 16 || key[1] == KEY_UP
                cursor = cursor -1
            end
            if key[1] == 4 || key[1] == KEY_PAGEDOWN
                cursor = cursor + mgetmaxy / 2
            end
            if key[1] == 21 || key[1] == KEY_PAGEUP
                cursor = cursor - mgetmaxy / 2
            end
            if key[1] == 12
                mclear
                mrefresh
            end

            if key[1] == KEY_q || key[1] == 3
                lines2 = []
                cursor = 0
                scroll_top = 0
                break;
            end
            if key[1] == 27 || key[1] == 7
                break;
            end
            
            if key[1] == 10 || key[1] == 13
                array = lines2[cursor].split(':')

                if !File.exist?(array[0])
                    output = msystem("find . -name #{array[0]}")
                    array2 = output.split
                    
                    if output.size != 0 && array2.size == 1
                        array[0] = output.chomp
                    end
                    if array2.size >  1
                        mclear()                    
                        mmvprintw(0,0, "find two or more " + array[1])
                        mrefresh()
                        mgetch()
                        
                        #array[0] = array2[0].chomp
                    end
                end

                if File.exist?(array[0]) && array[1].to_i != 0
                    # array[1] == line number
                    # array[0] == file name

                    mendwin()

                    system("#{editor} #{line_option}" + array[1] + " " + array[0])

                    minitscr()
                end
            end

            ### 修正 ###
            if cursor < 0
                cursor = 0
            end
            if cursor >= lines2.size
                cursor = lines2.size - 1
            end
            if cursor < scroll_top
                 scroll_top = cursor
            end
            if cursor > scroll_top + mgetmaxy() - 1
                scroll_top = scroll_top + (cursor-scroll_top-mgetmaxy()) + 1
            end
        end
    end

    reread()
end

###############################################################################
# grepのラッパー
###############################################################################
def mgrep(grep_cmd, editor, line_option)
    if !mis_curses
        puts "curses is not running..."
        return
    end

    lines = []
    cursor = 0
    scroll_top = 0

    ### grepを実行 ###
    mclear()
    mmvprintw(0,0, "grepping...")
    mrefresh()

    mendwin()
    set_signal_clear()
    if !system(grep_cmd)
        set_signal_mfiler2()
        minitscr()
        return
    end

    set_signal_mfiler2()
    minitscr()

    f = open("/tmp/mfiler_out_of_grep", "r")
    output = f.read()
    f.close()

    if output == ""
        mmvprintw(0,0,"no output")
        mrefresh()
        mgetch()
        return
    end

    ### grepの出力一行ごとにlinesに格納 ###
    i = 0
    tmp = ""
    while i < output.size()
        if output[i] == ?\n
            if tmp != ""
#                if lines.size < 500
                    lines.push(tmp)
#                end
            end
            tmp = ""
        else
            tmp = tmp + output[i].chr
        end

        i = i + 1
    end

    ### メインループ ###
    maxx = mgetmaxx()

    if lines.size > 0 
        key = [0, 0]
        while true
            ### view ###
            mclear()
            n = scroll_top
            while n < lines.size
                if cursor == n
                    mattron(MA_REVERSE)
                end

                line = wstrcut(lines[n], maxx-1)
                mmvprintw(n - scroll_top , 0, "%s", line)

                if cursor == n
                    mattroff()
                end
                
                n = n + 1
            end
            
            mrefresh()

            ### input ###
            key = mgetch()

            if key[1] == 14 || key[1] == KEY_DOWN
                cursor = cursor + 1
            end
            if key[1] == 16 || key[1] == KEY_UP
                cursor = cursor -1
            end
            if key[1] == 4 || key[1] == KEY_PAGEDOWN
                cursor = cursor + mgetmaxy / 2
            end
            if key[1] == 21 || key[1] == KEY_PAGEUP
                cursor = cursor - mgetmaxy / 2
            end
            if key[1] == 12
                mclear
                mrefresh
            end

            if key[1] == ?q || key[1] == 3
                lines = []
                cursor = 0
                scroll_top = 0
                break;
            end
            if key[1] == 27 || key[1] == 7
                break;
            end
            
            if key[1] == 10 || key[1] == 13
                array = lines[cursor].split(':')

                if !File.exist?(array[0])
                    output = msystem("find . -name #{array[0]}")
                    array2 = output.split
                    
                    if output.size != 0 && array2.size == 1
                        array[0] = output.chomp
                    end
                    if array2.size >  1
                        mclear()                    
                        mmvprintw(0,0, "find two or more " + array[1])
                        mrefresh()
                        mgetch()
                        
                        #array[0] = array2[0].chomp
                    end
                end

                if File.exist?(array[0]) && array[1].to_i != 0
                    # array[1] == line number
                    # array[0] == file name

                    mendwin()

                    system(editor + " " + line_option + array[1] + " " + array[0])

                    minitscr()
                end
            end

            ### modification ###
            if cursor < 0
                cursor = 0
            end
            if cursor >= lines.size
                cursor = lines.size - 1
            end
            if cursor < scroll_top
                 scroll_top = cursor
            end
            if cursor > scroll_top + mgetmaxy() - 1
                scroll_top = scroll_top + (cursor-scroll_top-mgetmaxy()) + 1
            end
        end
    end
end

###############################################################################
# プレイリスト
###############################################################################
def playlist_start_hook_ldir()
    $playlist_focusback_ldir = view_focusback(0)
    set_view_focusback(0, true)
    $playlist_sort_ldir = sort_kind(0)
    set_sort_kind(0, "none")
    $playlist_sort_dir_up_ldir = sort_dir_up(0)
    set_sort_dir_up(0, false)
end

def playlist_start_hook_rdir()
    $playlist_focusback = view_focusback(1)
    set_view_focusback(1, true)
    $playlist_sort_rdir = sort_kind(1)
    set_sort_kind(1, "none")
    $playlist_sort_dir_up_rdir = sort_dir_up(1)
    set_sort_dir_up(1, false)
end

def playlist_end_hook_ldir()
    set_view_focusback(0, $playlist_focusback_ldir)
    set_sort_kind(0, $playlist_sort_ldir)
    set_sort_dir_up(0, $playlist_sort_dir_up_ldir)
end

def playlist_end_hook_rdir()
    set_view_focusback(1, $playlist_focusback_rdir)
    set_sort_kind(1, $playlist_sort_rdir)
    set_sort_dir_up(1, $playlist_sort_dir_up_rdir)
end

def playlist_start(fname)
    if is_adir_left
        if $playlist_ldir == nil
            $playlist_pwd_ldir = adir_path()
            $playlist_scrolltop_ldir = scrolltop_ldir()
            $playlist_cursor_ldir = cursor_ldir()

            playlist_start_hook_ldir()

            set_scrolltop_ldir(0)
            cursor_move_ldir(1)

            begin
                f = open(fname, "r")
                $playlist_ldir = f.read()
                f.close
            rescue
                puts fname
                exit(1)
                return false
            end

            ### 絶対パスのファイルリストを相対パスのファイルリストに変換 ###
            new_playlist_ldir = ""
            path2 = adir_path()

            $playlist_ldir.each() do |fname|
                fname.chomp!()
                
                path1 = fname

                n = path2.size - 1
                while n >= 0
                    if path1.include?(path2[0..n])
                        same = path2[0..n]
                        diff = path1[n+1..-1]
                        break
                    end

                    n = n -1
                end

                new_playlist_ldir += "../" * path2[same.size..-1].scan(/\//).size + diff
                new_playlist_ldir += "\n"
            end

            $playlist_ldir = new_playlist_ldir

            reread_ldir()
        end
    else 
        if $playlist_rdir == nil
            $playlist_pwd_rdir = adir_path()
            $playlist_scrolltop_rdir = scrolltop_rdir()
            $playlist_cursor_rdir = cursor_rdir()

            playlist_start_hook_rdir()

            set_scrolltop_rdir(0)
            cursor_move_rdir(1)

            begin
                f = open(fname, "r")
                $playlist_rdir = f.read()
                f.close
            rescue
                puts fname
                exit(1)
                return false
            end

            ### 絶対パスのファイルリストを相対パスのファイルリストに変換 ###
            new_playlist_rdir = ""
            path2 = adir_path()

            $playlist_rdir.each() do |fname|
                fname.chomp!()
                
                path1 = fname

                n = path2.size - 1
                while n >= 0
                    if path1.include?(path2[0..n])
                        same = path2[0..n]
                        diff = path1[n+1..-1]
                        break
                    end

                    n = n -1
                end

                new_playlist_rdir += "../" * path2[same.size..-1].scan(/\//).size + diff
                new_playlist_rdir += "\n"
            end

            $playlist_rdir = new_playlist_rdir

            reread_rdir()
        end
    end
end

def playlist_start2(fname)
    if is_adir_left
        if $playlist_ldir == nil
            $playlist_pwd_ldir = adir_path()
            $playlist_scrolltop_ldir = scrolltop_ldir()
            $playlist_cursor_ldir = cursor_ldir()

            playlist_start_hook_ldir()

            dir_move_ldir("/")

            begin
                f = open(fname, "r")
                $playlist_ldir = f.read()
                f.close
            rescue
                return false
            end

            reread_ldir()
        end
    else 
        if $playlist_rdir == nil
            $playlist_pwd_rdir = adir_path()
            $playlist_scrolltop_rdir = scrolltop_rdir()
            $playlist_cursor_rdir = cursor_rdir()

            playlist_start_hook_rdir()

            dir_move_rdir("/")

            begin
                f = open(fname, "r")
                $playlist_rdir = f.read()
                f.close
            rescue
                return false
            end

            reread_rdir()
        end
    end
end

def make_new_playlist(fname)
    if !File.exist?(adir_path() + fname)
        begin
            f = File.new(adir_path() + fname,File::CREAT|File::TRUNC|File::WRONLY)
            f.close
        rescue
            return false
        end
        reread()
        cursor_move(file_num(fname))

        $making_playlist_path = adir_path() + fname
    end
end

def add_cursor_file_to_making_playlist()
    if $making_playlist_path == nil
        message("$making_playlist_path is nil")
    else
        if marking
            f = open($making_playlist_path, "a")
            adir_mark_fullpath.each do |path|
                f.puts path
            end

            mark_clear()
            f.close

            message("add marking files to " + $making_playlist_path)
        else
            f = open($making_playlist_path, "a")
            f.puts cursor_path()
            f.close

            message("add " + cursor_path + " to " + $making_playlist_path)
        end
    end
end

def change_making_playlist()
    if cursor_ext() == "list"
        $making_playlist_path = cursor_path()
        message('setted ' + cursor_path + ' as $making_playlist_path')
    else
        message("cursor file is not playlist")
    end
end

###############################################################################
# sftpクライアント
###############################################################################
require 'net/ssh'
require 'net/sftp'

$sftp_path = nil
$sftp_host = nil
$sftp_user = nil

def sftp_start(host, user, passwd, path)
    if path[-1] != "/"[0]
        path = path + "/"
    end

    $sftp_path = path
    $sftp_host = host
    $sftp_user = user
    $sftp_passwd = passwd

    sftp_path_refresh()

    $sftp_view_tmp = view_filer()
    set_view_filer("all")
    reread_rdir()
end

def sftp_start2(host, user, path)
    if path[-1] != "/"[0]
        path = path + "/"
    end

    $sftp_path = path
    $sftp_host = host
    $sftp_user = user
    $sftp_passwd = nil

    sftp_path_refresh()

    $sftp_view_tmp = view_filer()
    set_view_filer("all")
    reread_rdir()
end

def sftp_end()
    $sftp_path = nil
    $sftp_host = nil
    $sftp_usre = nil

    $view_path_rdir = nil

    eval("view_" + $sftp_view_tmp)
    reread_rdir()
end

def sftp_path_refresh()
    $view_path_rdir = "sftp://" + $sftp_user + "@" + $sftp_host + ":" + $sftp_path + rdir_mask()
end

def sftp_change_dir(path)
    if path[-1] != "/"[0]
        path = path + "/"
    end

    $sftp_path = path

    sftp_path_refresh()

    cursor_move_rdir(0)
    set_scrolltop_rdir(0)

    reread_rdir()

    mark_clear()
end

def sftp_read(sftp)
    h = sftp.opendir($sftp_path)
    sftp.readdir( h ).each do |item|
        if item.filename != "."
            permissions = "%06o" % item.attributes.permissions
            kind = -1
            if permissions.index("10") == 0
                kind = 0    # ファイル
            elsif permissions.index("04") == 0
                kind = 1    # ディレクトリ
            elsif permissions.index("12") == 0
                kind = 2    # リンク
            end

            permissions = Integer(permissions[2..-1])
            if permissions & 0400 != 0
                user_r = true
            else
                user_r = false
            end
            if permissions & 0200 != 0
                user_w = true
            else
                user_w = false
            end
            if permissions & 0100 != 0
                user_x = true
            else
                user_x = false
            end

            if permissions & 0040 != 0
                group_r = true
            else
                group_r = false
            end
            if permissions & 0020 != 0
                group_w = true
            else
                group_w = false
            end
            if permissions & 0010 != 0
                group_x = true
            else
                group_x = false
            end

            if permissions & 0004 != 0
                other_r = true
            else
                other_r = false
            end
            if permissions & 0002 != 0
                other_w = true
            else
                other_w = false
            end
            if permissions & 0001 != 0
                other_x = true
            else
                other_x = false
            end

            user = item.longname.split[2]
            group = item.longname.split[3]

            linkto = ""

            user_rwx = [user_r, user_w, user_x]
            group_rwx = [group_r, group_w, group_x]
            other_rwx = [other_r, other_w, other_x]

            add_file2_rdir(item.filename, linkto, kind, user_rwx, group_rwx, other_rwx, item.attributes.size, item.attributes.mtime, user, group)
        end
    end
    sftp.close_handle( h )
end

def sftp_disk_read_hook_rdir()
    if $sftp_passwd == nil
        Net::SFTP.start($sftp_host, $sftp_user, :host_key=>"ssh-rsa") do |sftp|
            sftp_read(sftp)
        end
    else
        Net::SFTP.start($sftp_host, $sftp_user, $sftp_passwd) do |sftp|
            sftp_read(sftp)
        end
    end
end


SFTP_OVERRIDE_NIL = 0
SFTP_OVERRIDE_NO_ALL = 1
SFTP_OVERRIDE_YES_ALL = 2

def sftp_copy_dir(sftp, path)
    h = sftp.opendir(path)
    sftp.readdir( h ).each do |item|
        if item.filename != "." && item.filename != ".."
            permissions = "%06o" % item.attributes.permissions
            kind = -1
            if permissions.index("10") == 0
                kind = 0    # ファイル
            elsif permissions.index("04") == 0
                kind = 1    # ディレクトリ
            end

            permissions = Integer(permissions[2..-1])

            if kind == 0
                message_nonstop(path + item.filename)
                sftp.get_file(path + item.filename, item.filename)
                File.new(item.filename).chmod(permissions)
                File.utime(0, item.attributes.mtime, item.filename)
            elsif kind == 1
                Dir.mkdir(item.filename)

                cwd = Dir.pwd()
                Dir.chdir(item.filename)
                sftp_copy_dir(sftp, path + item.filename + "/")
                Dir.chdir(cwd)

                File.new(item.filename).chmod(permissions)
                File.utime(0, item.attributes.mtime, item.filename)
            end
        end
    end
    sftp.close_handle( h )
end

def sftp_copy_dir2(sftp, path)
    dir = Dir.open(".")
    dir.each() do |item|
        if item != "." && item != ".."
            if !File.directory?(item)
                sftp.put_file(item, path + item)
                sftp.setstat(path + item, :permissions => Integer(("%o" % File.stat(item).mode)[2..-1]))
                #sftp.setstat(path + item, :mtime => File.stat(item).mtime)
            else
                sftp.mkdir(path + item, permissions => Integer(("%o" % File.stat.new(item).mode)[2..-1]))
                #sftp.setstat(path + item, :mtime => File.stat(item).mtime)

                cwd = Dir.pwd()
                Dir.chdir(item)
                sftp_copy_dir2(sftp, path + item + "/")
                Dir.chdir(cwd)
            end
        end
    end
    dir.close()
end

def sftp_get_main(sftp, file)
    if File.exist?(file)
        if $sftp_override == SFTP_OVERRIDE_YES_ALL
            ret = 1
        elsif $sftp_override == SFTP_OVERRIDE_NO_ALL
            ret = 0
        else
            ret = select_string(adir_path + file + " override?", ["No", "Yes", "No(all)", "Yes(all)", "Cancel"], 4)
        end

        if ret == 1 || ret == 3
            kind = file_kind_rdir(file_num(file))

            if kind == 1     # ディレクトリ
                begin
                    FileUtils.rm_r(file)

                    Dir.mkdir(file)

                    cwd = Dir.pwd()
                    Dir.chdir(file)
                    sftp_copy_dir(sftp, $sftp_path + file + "/")
                    Dir.chdir(cwd)

                    File.new(file).chmod(file_permission_rdir(file_num_rdir(file)))
                    File.utime(0, file_mtime_rdir(file_num_rdir(file)), file)
                rescue
                end
            elsif kind == 2     # リンク
            elsif kind == 3     # キャラクターデバイス
            elsif kind == 4     # ブロックデバイス
            elsif kind == 5     # FIFO
            elsif kind == 6     # ソケット
            else                # ファイル
                begin
                    message_nonstop($sftp_path + file)

                    FileUtils.rm_f(file)

                    sftp.get_file($sftp_path + file, file)
                    File.new(file).chmod(file_permission_rdir(file_num_rdir(file)))
                    File.utime(0, file_mtime_rdir(file_num_rdir(file)), file)
                rescue
                end
            end
        end

        if ret == 2
            $sftp_override = SFTP_OVERRIDE_NO_ALL
        elsif ret == 3
            $sftp_override = SFTP_OVERRIDE_YES_ALL
        end
    else
        kind = file_kind_rdir(file_num(file))

        if kind == 1     # ディレクトリ
            begin
                Dir.mkdir(file)

                cwd = Dir.pwd()
                Dir.chdir(file)
                sftp_copy_dir(sftp, $sftp_path + file + "/")
                Dir.chdir(cwd)

                File.new(file).chmod(file_permission_rdir(file_num_rdir(file)))
                File.utime(0, file_mtime_rdir(file_num_rdir(file)), file)
            rescue
            end
        elsif kind == 2     # リンク
        elsif kind == 3     # キャラクターデバイス
        elsif kind == 4     # ブロックデバイス
        elsif kind == 5     # FIFO
        elsif kind == 6     # ソケット
        else                # ファイル
            begin
                sftp.get_file($sftp_path + file, file)
                File.new(file).chmod(file_permission_rdir(file_num_rdir(file)))
                File.utime(0, file_mtime_rdir(file_num_rdir(file)), file)
            rescue
            end
        end
    end
end

def sftp_get(path)
    cwd = Dir.pwd()
    Dir.chdir(path)

    if $sfpt_passwd
        Net::SFTP.start($sftp_host, $sftp_user, $sftp_passwd) do |sftp|
            rdir_mark().each() do |file|
                cursor_move(file_num(file))
                sftp_get_main(sftp, file)

                mark_off(file_num(file))
                mfiler_view()
                mrefresh()
            end
        end
    else
        Net::SFTP.start($sftp_host, $sftp_user, :host_key=>"ssh-rsa") do |sftp|
            rdir_mark().each() do |file|
                cursor_move(file_num(file))
                sftp_get_main(sftp, file)

                mark_off(file_num(file))
                mfiler_view()
                mrefresh()
            end
        end
    end

    Dir.chdir(cwd)

    reread_ldir()
end

def sftp_put_main(sftp, file)
    if file_num_rdir(file) != -1
        if $sftp_override == SFTP_OVERRIDE_YES_ALL
            ret = 1
        elsif $sftp_override == SFTP_OVERRIDE_NO_ALL
            ret = 0
        else
            ret = select_string($sftp_path + file + " override?", ["No", "Yes", "No(all)", "Yes(all)", "Cancel"], 4)
        end

        if ret == 1 || ret == 3
            kind = file_kind_ldir(file_num(file))

            if kind == 1     # ディレクトリ
                begin
                    sftp_rmdir_r(sftp, $sftp_path + file + "/")

                    sftp.mkdir($sftp_path + file, :permissions=>file_permission_ldir(file_num_ldir(file)))
                    #sftp.setstat($sftp_path + file, :mtime => file_mtime_ldir(file_num_ldir(file)))

                    cwd = Dir.pwd()
                    Dir.chdir(file)
                    sftp_copy_dir2(sftp, $sftp_path + file + "/")
                    Dir.chdir(cwd)
                rescue
                end
            elsif kind == 2     # リンク
            elsif kind == 3     # キャラクターデバイス
            elsif kind == 4     # ブロックデバイス
            elsif kind == 5     # FIFO
            elsif kind == 6     # ソケット
            else                # ファイル
                begin
                    sftp.remove(file)

                    sftp.put_file(file, $sftp_path + file)
                    #sftp.setstat($sftp_path + file, :mtime => file_mtime_ldir(file_num_ldir(file)))
                    sftp.setstat($sftp_path + file, :permissions => file_permission_ldir(file_num_ldir(file)))
                rescue
                end
            end
        end

        if ret == 2
            $sftp_override = SFTP_OVERRIDE_NO_ALL
        elsif ret == 3
            $sftp_override = SFTP_OVERRIDE_YES_ALL
        end
    else
        kind = file_kind_ldir(file_num(file))

        if kind == 1     # ディレクトリ
            sftp.mkdir($sftp_path + file, :permissions=>file_permission_ldir(file_num_ldir(file)))
            #sftp.setstat($sftp_path + file, :mtime => file_mtime_ldir(file_num_ldir(file)))

            cwd = Dir.pwd()
            Dir.chdir(file)
            sftp_copy_dir2(sftp, $sftp_path + file + "/")
            Dir.chdir(cwd)
        elsif kind == 2     # リンク
        elsif kind == 3     # キャラクターデバイス
        elsif kind == 4     # ブロックデバイス
        elsif kind == 5     # FIFO
        elsif kind == 6     # ソケット
        else                # ファイル
            sftp.put_file(file, $sftp_path + file)
            sftp.setstat($sftp_path + file, :permissions => file_permission_ldir(file_num_ldir(file)))
            #sftp.setstat($sftp_path + file, :mtime => file_mtime_ldir(file_num_ldir(file)))
        end
    end
end

def sftp_put(path)
    $sftp_override = SFTP_OVERRIDE_NIL

    if $sftp_passwd
        Net::SFTP.start($sftp_host, $sftp_user, $sftp_passwd) do |sftp|
            ldir_mark().each() do |file|
                cursor_move(file_num(file))
                sftp_put_main(sftp, file)

                mark_off(file_num(file))
                mfiler_view()
                mrefresh()
            end
        end
    else
        Net::SFTP.start($sftp_host, $sftp_user, :host_key=>"ssh-rsa") do |sftp|
            ldir_mark().each() do |file|
                cursor_move(file_num(file))
                sftp_put_main(sftp, file)

                mark_off(file_num(file))
                mfiler_view()
                mrefresh()
            end
        end
    end

    reread_rdir()
end

def sftp_get_and_remove(path)
    cwd = Dir.pwd()
    Dir.chdir(path)

    if $sfpt_passwd
        Net::SFTP.start($sftp_host, $sftp_user, $sftp_passwd) do |sftp|
            rdir_mark().each() do |file|
                kind = file_kind_rdir(file_num_rdir(file))

                cursor_move(file_num(file))
                sftp_get_main(sftp, file)

                if kind == 1
                    sftp_rmdir_r(sftp, $sftp_path + file + "/")
                else
                    sftp.remove($sftp_path + file)
                end

                mark_off(file_num(file))
                mfiler_view()
                mrefresh()
            end
        end
    else
        Net::SFTP.start($sftp_host, $sftp_user, :host_key=>"ssh-rsa") do |sftp|
            rdir_mark().each() do |file|
                kind = file_kind_rdir(file_num_rdir(file))

                cursor_move(file_num(file))
                sftp_get_main(sftp, file)

                if kind == 1
                    sftp_rmdir_r(sftp, $sftp_path + file + "/")
                else
                    sftp.remove($sftp_path + file)
                end

                mark_off(file_num(file))
                mfiler_view()
                mrefresh()
            end
        end
    end

    Dir.chdir(cwd)

    reread_ldir()
    reread_rdir()
end

def sftp_put_and_remove()
    $sftp_override = SFTP_OVERRIDE_NIL

    if $sfpt_passwd
        Net::SFTP.start($sftp_host, $sftp_user, $sftp_passwd) do |sftp|
            ldir_mark().each() do |file|
                kind = file_kind_ldir(file_num_ldir(file))

                cursor_move(file_num(file))
                sftp_put_main(sftp, file)

                FileUtils.rm_rf(file)

                mark_off(file_num(file))
                mfiler_view()
                mrefresh()
            end
        end
    else
        Net::SFTP.start($sftp_host, $sftp_user, :host_key=>"ssh-rsa") do |sftp|
            ldir_mark().each() do |file|
                kind = file_kind_ldir(file_num_ldir(file))

                cursor_move(file_num(file))
                sftp_put_main(sftp, file)

                FileUtils.rm_rf(file)

                mark_off(file_num(file))
                mfiler_view()
                mrefresh()
            end
        end
    end

    reread_ldir()
    reread_rdir()
end

def sftp_rmdir_r(sftp, path)
    h = sftp.opendir(path)
    sftp.readdir( h ).each do |item|
        if item.filename != "." && item.filename != ".."
            permissions = "%06o" % item.attributes.permissions
            kind = -1
            if permissions.index("10") == 0
                kind = 0    # ファイル
            elsif permissions.index("04") == 0
                kind = 1    # ディレクトリ
            end

            if kind == 1
                sftp_rmdir_r(sftp, path + item.filename + "/")
            else
                sftp.remove(path + item.filename)
            end
        end
    end
    sftp.close_handle( h )
    sftp.rmdir(path)
end
###############################################################################
# ファイラーの状態をセーブ／読み込み
###############################################################################
def save_filer_stat()
    f = open(ENV['HOME'] + "/.mfinfo", "w")

    f.puts "<Left dir path>"
    f.puts ldir_path()
    f.puts "<Right dir path>"
    f.puts rdir_path()

    f.puts "<tab>"
    f.puts tab_max()

    i = 0
    while i < tab_max()
        f.puts tab_path(i)
        f.puts tab_scrolltop(i)
        f.puts tab_cursor(i)

        i+=1
    end

    f.puts "<Left Dir Sort Kind>"
    f.puts sort_kind(0)

    f.puts "<Right Dir sort Kind>"
    f.puts sort_kind(1)

    f.puts "<Left Sort dir up>"
    f.puts sort_dir_up(0)

    f.puts "<Right Sort dir up>"
    f.puts sort_dir_up(1)

    f.puts "<Left Dir Name only>"
    f.puts view_nameonly(0)

    f.puts "<Right Dir Name only>"
    f.puts view_nameonly(1)

    f.puts "<Left Dir Focus Back>"
    f.puts view_focusback(0)

    f.puts "<Right Dir Focus Back>"
    f.puts view_focusback(1)

    f.puts "<Left Dir Remove Dir>"
    f.puts view_removedir(0)

    f.puts "<Right Dir Remove Dir>"
    f.puts view_removedir(1)

    f.puts "<Filer mode>"
    f.puts view_filer()

    f.puts "<color>"
    f.puts view_color()

    f.puts "<kanjicode file name>"
    f.puts kanjicode_filename()

    f.puts "<directory menu>"
    save_directory_menu(f)

    f.puts "<default dir>"
    f.puts $default_one_dir

    f.puts "<view option mtime>"
    f.puts view_mtime()
    f.puts "<view option size>"
    f.puts view_size()
    f.puts "<view option user>"
    f.puts view_user()
    f.puts "<view option group>"
    f.puts view_group()
    f.puts "<view option nlink>"
    f.puts view_nlink()
    f.puts "<view option permission>"
    f.puts view_permission()
    f.puts "<view fname divide ext>"
    f.puts view_fname_divide_ext()

    f.close
end

def load_filer_stat()
    if !File.exist?(ENV['HOME'] + "/.mfinfo")
        return 
    end

    begin
        f = open(ENV['HOME'] + "/.mfinfo", "r")

        f.readline
        line = f.readline.chomp!
        dir_move_ldir(line)

        f.readline
        line = f.readline.chomp!
        dir_move_rdir(line)

        f.readline
        max = f.readline.chomp!.to_i

        i = 0
        while i < max
            path = f.readline.chomp!
            scrolltop = f.readline.chomp!.to_i
            cursor = f.readline.chomp!.to_i

            tab_new(i, path, scrolltop, cursor)
            i += 1
        end

        f.readline
        line = f.readline.chomp!()
        set_sort_kind(0, line)

        f.readline
        line = f.readline.chomp!()
        set_sort_kind(1, line)

        f.readline
        line = f.readline.chomp!() == "true" ? true:false
        set_sort_dir_up(0, line)

        f.readline
        line = f.readline.chomp!() == "true" ? true:false
        set_sort_dir_up(1, line)

        f.readline
        line = f.readline.chomp!() == "true" ? true:false
        set_view_nameonly(0, line)

        f.readline
        line = f.readline.chomp!() == "true" ? true:false
        set_view_nameonly(1, line)

        f.readline
        line = f.readline.chomp!() == "true" ? true:false
        set_view_focusback(0, line)

        f.readline
        line = f.readline.chomp!() == "true" ? true:false
        set_view_focusback(1, line)

        f.readline
        line = f.readline.chomp!() == "true" ? true:false
        set_view_removedir(0, line)

        f.readline
        line = f.readline.chomp!() == "true" ? true:false
        set_view_removedir(1, line)

        f.readline
        line = f.readline.chomp!()
        set_view_filer(line)

        f.readline
        line = f.readline.chomp!() == "true" ? true:false
        set_view_color(line)

        f.readline
        line = f.readline.chomp!()
        set_kanjicode_filename(line)

        f.readline
        read_directory_menu(f)

        f.readline
        $default_one_dir = f.readline.chomp()

        f.readline
        line = f.readline.chomp!() == "true" ? true: false
        set_view_mtime(line)

        f.readline
        line = f.readline.chomp!() == "true" ? true: false
        set_view_size(line)

        f.readline
        line = f.readline.chomp!() == "true" ? true: false
        set_view_user(line)

        f.readline
        line = f.readline.chomp!() == "true" ? true: false
        set_view_group(line)

        f.readline
        line = f.readline.chomp!() == "true" ? true: false
        set_view_nlink(line)

        f.readline
        line = f.readline.chomp!() == "true" ? true: false
        set_view_permission(line)

        f.readline
        line = f.readline.chomp!() == "true" ? true: false
        set_view_fname_divide_ext(line)
    rescue
    end
end

load_filer_stat()

###############################################################################
# 互換性のための定義
###############################################################################
def option_kanjicode(code)
    set_kanjicode code
end

def option_kanjicode_filename(code)
    set_kanjicode_filename(code)
end

def option_bold_exe(a)
    set_view_bold_exe a
end

def option_bold_dir(a)
    set_view_bold_dir a
end

def option_color_mark(color)
    set_view_color_mark color
end

def option_color_exe(color)
    set_view_color_exe color
end

def option_color_dir(color)
    set_view_color_dir color
end

def option_color_link(color)
    set_view_color_link color
end

