#include "common.h"
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>

vector_obj* gJobs;      // vector of sJob
sJob* gForgroundJob = NULL;    // フォアグランドジョブ

static void wait_forground_job();       // フォアグランドジョブの終了を待つ

///////////////////////////////////////////////////////////////////////////////
// 環境変数の展開
///////////////////////////////////////////////////////////////////////////////
static void extend_env(char* cmdline, string_obj* out)
{
    bool squote = false;
    bool dquote = false;
    
    char* p = cmdline;
    while(*p) {
        if(*p == '\'') {
            p++;

            if(!dquote) squote = !squote;

            string_push_back(out, "\'");
        }
        else if(*p == '"') {
            p++;

            if(!squote) dquote = !dquote;

            string_push_back(out, "\"");
        }
        else if(!squote && !dquote && *p == '~' && (*(p+1) == 0 || *(p+1) == ' ' || *(p+1) == '/')) {
            p++;
            
            char* home = getenv("HOME");
            if(home == NULL) {
                fprintf(stderr, "$HOME is null");
                exit(1);
            }
            
            string_push_back(out, home);
        }
        else if(!squote && *p == '$' && *(p+1) != '(') {
            p++;
            
            char name[1024];
            char* p2 = name;
            while(*p>='a' && *p<='z' || *p>='A' && *p<='Z' || *p=='_' || *p>='0' && *p<='9') {
                *p2++ = *p++;
            }
            *p2 = 0;
            
            char* env = getenv(name);
            
            if(env) string_push_back(out, env);
        }
        else {
            char tmp[2];
            tmp[0] = *p++;
            tmp[1] = 0;
            string_push_back(out, tmp);
        }
    }
}

///////////////////////////////////////////////////////////////////////////////
// パーサー
///////////////////////////////////////////////////////////////////////////////
enum eRedirect { kRedirectInput, kRedirectOverwrite, kRedirectAppend };

struct sRedirect {
    enum eRedirect mType;
    int mFd;
    char mFName[1024];
};

struct sStatment {
    vector_obj* argvs;
    vector_obj* redirections_list;
    bool background;
    
    sStatment() {
        argvs = vector_new(10);
        redirections_list = vector_new(10);
        background = false;
    }
    
    ~sStatment() {
        for(int i=0; i<vector_size(redirections_list); i++) {
            vector_obj* redirections = (vector_obj*)vector_item(redirections_list, i);
            
            for(int j=0; j<vector_size(redirections); j++) {
                delete (sRedirect*)vector_item(redirections, j);
            }
            
            vector_delete(redirections);
        }
        vector_delete(redirections_list);
        
        for(int i=0; i<vector_size(argvs); i++) {
            vector_obj* argv = (vector_obj*)vector_item(argvs, i);
            
            for(int j=0; j<vector_size(argv); j++) {
                string_delete((string_obj*)vector_item(argv, j));
            }
            
            vector_delete(argv);
        }
        vector_delete(argvs);
    }
};

static bool parse(char* cmdline, vector_obj* statments)
{
    char* p = cmdline;
    while(*p == ' ' || *p == '\t') {
        p++;
    }
    
    sStatment* statment = new sStatment;
    vector_obj* argv = vector_new(10);
    vector_obj* redirections = vector_new(10);

    char buf[8096];
    char* p2 = buf;
    bool squote = false;
    bool dquote = false;
    bool next_is_rfname = false;
    sRedirect* redirection;
    
    while(*p) {
        if(!dquote && *p == '\'') {
            p++;
            squote = !squote;
        }
        else if(!squote && *p == '"') {
            p++;
            dquote = !dquote;
        }
        else if(squote || dquote) {
            *p2++ = *p++;
        }
        else if(*p == '\\') {
            p++;

            if(*p) *p2++ = *p++;
        }
        else if(*p == '$' && *(p+1) == '(') {
            p++;
            p++;
            char cmd[4096];

            char* p3 = cmd;
            *p3++ = '`';
            while(*p != ')') {
                if(*p == 0) {
                    gErrMsgCancel = false;
                    err_msg("expected `");
                    return false;
                }
                *p3++ = *p++;
            }
            sprintf(p3, "`.chomp");
            p++;
            int error;
            VALUE value = rb_eval_string_protect(cmd, &error);
            if(error) {
                switch(error) {
                case TAG_RETURN:
                    fprintf(stderr, "unexpected return\n");
                    break;
                    
                case TAG_NEXT:
                    fprintf(stderr, "unexpected next\n");
                    break;
                    
                case TAG_BREAK:
                    fprintf(stderr, "unexpected break\n");
                    break;
                    
                case TAG_REDO:
                    fprintf(stderr, "unexpected redo\n");
                    break;
                    
                case TAG_RETRY:
                    fprintf(stderr, "retry outside of rescue clause\n");
                    break;
                    
                case TAG_RAISE:
                case TAG_FATAL:
                    fprintf(stderr, "%s: %s\n",
                        rb_class2name(CLASS_OF(ruby_errinfo)),
                        RSTRING(rb_obj_as_string(ruby_errinfo))->ptr);
                    break;
                
                default:
                    fprintf(stderr, "unknown longjmp status %d\n", error);
                    break;
                }
            }
            else {
                Check_Type(value, T_STRING);
                char* p4 = RSTRING(value)->ptr;
                if(p4 == NULL) {
                    return false;
                }
                else {
                    while(*p4) {
                        *p2++ = *p4++;
                    }
                }
            }
        }
        else if(*p == '`') {
            p++;
            char cmd[4096];

            char* p3 = cmd;
            *p3++ = '`';
            while(*p != '`') {
                if(*p == 0) {
                    gErrMsgCancel = false;
                    err_msg("expected `");
                    return false;
                }
                *p3++ = *p++;
            }
            sprintf(p3, "`.chomp");
            p++;
            int error;
            VALUE value = rb_eval_string_protect(cmd, &error);
            if(error) {
                switch(error) {
                case TAG_RETURN:
                    fprintf(stderr, "unexpected return\n");
                    break;
                    
                case TAG_NEXT:
                    fprintf(stderr, "unexpected next\n");
                    break;
                    
                case TAG_BREAK:
                    fprintf(stderr, "unexpected break\n");
                    break;
                    
                case TAG_REDO:
                    fprintf(stderr, "unexpected redo\n");
                    break;
                    
                case TAG_RETRY:
                    fprintf(stderr, "retry outside of rescue clause\n");
                    break;
                    
                case TAG_RAISE:
                case TAG_FATAL:
                    fprintf(stderr, "%s: %s\n",
                        rb_class2name(CLASS_OF(ruby_errinfo)),
                        RSTRING(rb_obj_as_string(ruby_errinfo))->ptr);
                    break;
                
                default:
                    fprintf(stderr, "unknown longjmp status %d\n", error);
                    break;
                }
            }
            else {
                Check_Type(value, T_STRING);
                char* p4 = RSTRING(value)->ptr;
                if(p4 == NULL) {
                    return false;
                }
                else {
                    while(*p4) {
                        *p2++ = *p4++;
                    }
                }
            }
        }
        else if(*p == ' ' || *p == '\t') {
            while(*p == ' ' || *p == '\t') {
                p++;
            }

            *p2 = 0;
            if(strcmp(buf, "") != 0) {
                if(next_is_rfname) {
                    next_is_rfname = false;
                    strcpy(redirection->mFName, buf);
                }
                else
                    vector_add(argv, string_new(buf));
            }
            p2 = buf;
        }
        else if(*p == '&') {
            p++;
            statment->background = true;

            *p2 = 0;
            if(strcmp(buf, "") != 0) {
                if(next_is_rfname) {
                    next_is_rfname = false;
                    strcpy(redirection->mFName, buf);
                }
                else
                    vector_add(argv, string_new(buf));
            }
            p2 = buf;
        }
        else if(*p == '|') {
            p++;
            
            while(*p == ' ' || *p == '\t') {
                p++;
            }

            *p2 = 0;
            if(strcmp(buf, "") != 0) {
                if(next_is_rfname) {
                    next_is_rfname = false;
                    strcpy(redirection->mFName, buf);
                }
                else
                    vector_add(argv, string_new(buf));
            }
            p2 = buf;
            
            if(vector_size(argv) == 0) return false;
            vector_add(statment->argvs, argv);
            argv = vector_new(10);
            
            if(next_is_rfname) return false;
            vector_add(statment->redirections_list, redirections);
            redirections = vector_new(10);
        }
        else if( (*p == '1' && *(p+1) == '>') 
                    || (*p == '1' && *(p+1) == '>' && *(p+2) == '>') 
                    || (*p == '2' && *(p+1) == '>') 
                    || (*p == '2' && *(p+1) == '>' && *(p+2) == '>') 
                    || (*p == '>' && *(p+1) == '>')
                    || *p =='>' 
                    || *p =='<'
                    ) 
       {
            *p2 = 0;
            if(strcmp(buf, "") != 0) {
                if(next_is_rfname) {
                    next_is_rfname = false;
                    strcpy(redirection->mFName, buf);
                }
                else
                    vector_add(argv, string_new(buf));
            }
            p2 = buf;
            
            int fd;
            eRedirect type;
            
            if(*p == '1' && *(p+1) == '>') {
                p+=2;
                
                fd = 1;
                type = kRedirectOverwrite;
            }
            else if(*p == '1' && *(p+1) == '>' && *(p+2) == '>') {
                p+=3;
                
                fd = 1;
                type = kRedirectAppend;
            }
            else if(*p == '2' && *(p+1) == '>') {
                p+=2;
                
                fd = 2;
                type = kRedirectOverwrite;
            }
            else if(*p == '2' && *(p+1) == '>' && *(p+2) == '>') {
                p+=3;
                
                fd = 2;
                type = kRedirectAppend;
            }
            else if(*p == '>' && *(p+1) == '>') {
                p+=2;
                
                fd = 1;
                type = kRedirectAppend;
            }
            else if(*p =='>') {
                p++;
                
                fd = 1;
                type = kRedirectOverwrite;
            }
            else {
                p++;
                
                fd = 0;
                type = kRedirectInput;
            }
            
            while(*p == ' ' || *p == '\t') {
                p++;
            }
            
            redirection = new sRedirect;
            
            redirection->mType = type;
            redirection->mFd = fd;
            
            next_is_rfname = true;
            
            vector_add(redirections, redirection);
        }
        else if(*p == ';') {
            p++;
            
            while(*p == ' ' || *p == '\t') {
                p++;
            }

            *p2 = 0;
            if(strcmp(buf, "") != 0) {
                if(next_is_rfname) {
                    next_is_rfname = false;
                    strcpy(redirection->mFName, buf);
                }
                else
                    vector_add(argv, string_new(buf));
            }
            p2 = buf;
            
            if(vector_size(argv) == 0) return false;
            vector_add(statment->argvs, argv);
            argv = vector_new(10);
            
            if(next_is_rfname) return false;
            vector_add(statment->redirections_list, redirections);
            redirections = vector_new(10);
            
            vector_add(statments, statment);
            statment = new sStatment;
        }
        else {
            *p2++ = *p++;
        }
    }
    
    *p2 = 0;
    if(strcmp(buf, "") != 0) {
        if(next_is_rfname) {
            next_is_rfname = false;
            strcpy(redirection->mFName, buf);
        }
        else
            vector_add(argv, string_new(buf));
    }
    
    if(vector_size(argv) == 0) return false;
    vector_add(statment->argvs, argv);
    
    if(next_is_rfname) return false;
    vector_add(statment->redirections_list, redirections);
    
    vector_add(statments, statment);
    
    return true;
}

///////////////////////////////////////////////////////////////////////////////
// シェル
///////////////////////////////////////////////////////////////////////////////
void shell(char* cmdline, bool hitanykey, char* title)
{
    /// 環境変数の展開 ///
    string_obj* cmdline2 = string_new("");
    extend_env(cmdline, cmdline2);
    
    /// パース ///
    vector_obj* statments = vector_new(10);
    if(!parse(string_c_str(cmdline2), statments)) {
        gErrMsgCancel = false;
        err_msg("parser err");
        return;
    }
    
    string_delete(cmdline2);

    /// 端末のタイトルに名前を付ける ///
    if(gChangeTerminalTitle) {
        printf("%c]2;mfiler2 %s%c", 033, title, 007);
        fflush(stdout);
    }

    /// 実行 ///
    for(int h=0; h<vector_size(statments); h++) {
        sStatment* statment = (sStatment*)vector_item(statments, h);
        
        int nextin = 0;
        int nextout = 1;
        
        sJob* job = new sJob(title);
        vector_add(gJobs, job);

        job->mPGroup = -1;
        
        for(int i=0; i<vector_size(statment->argvs); i++) {
            vector_obj* argv = (vector_obj*)vector_item(statment->argvs, i);
            vector_obj* redirections = (vector_obj*)vector_item(statment->redirections_list, i);
            
            int pipefds[2];
            
            if(i == vector_size(statment->argvs)-1) {
                nextout = 1;
            }
            else {
                if(pipe(pipefds) < 0) {
                    perror("pipe");
                }
                nextout = pipefds[1];
            }

            /// fork ///
            pid_t pid = fork();
            if(pid < 0) {
                perror("fork");
                exit(1);
            }

            /// 子プロセスの処理 ///
            if(pid == 0) {

                signal(SIGTTIN, SIG_DFL);
                signal(SIGTTOU, SIG_DFL);
                signal(SIGINT, SIG_DFL);
                signal(SIGQUIT, SIG_DFL);
                signal(SIGTSTP, SIG_DFL);
                signal(SIGCHLD, SIG_DFL);
                signal(SIGWINCH, SIG_DFL);
/*
P((f, "子プロセス pid %d", getpid()));
P((f, "子プロセス ssid %d", tcgetsid(0)));
P((f, "子プロセス pgrp %d\n", tcgetpgrp(0)));
*/
                if(job->mPGroup == -1) {
                    pid_t pid = getpid();

                    /// 独自のプロセスグループに入るまで待つ ///
                    pid_t pgrp = getpgid(0);
                    while(pgrp != pid) {
                        pgrp = getpgid(0);
                    }

                    // このプロセスグループのプロセスがフォアグランドに出るまで待つ //
                    if(!statment->background) {
                        pid_t pgrp = tcgetpgrp(0);
                        while(pgrp != pid) {
                            pgrp = tcgetpgrp(0);
                        }
                    }
                }
                else {
                    /// 独自のプロセスグループに入るまで待つ ///
                    pid_t pgrp = getpgid(0);
                    while(pgrp != job->mPGroup) {
                        pgrp = getpgid(0);
                    }
                }

/*
P((f, "子プロセス 2 pid %d", getpid()));
P((f, "子プロセス 2 ssid %d", tcgetsid(0)));
P((f, "子プロセス 2 pgrp %d\n", tcgetpgrp(0)));
*/

                /// パイプ ///
                if(nextin != 0) {
                    if(dup2(nextin, 0) < 0) {
                        perror("dup2");
                    }
                    close(nextin);
                }
                
                if(nextout != 1) {
                    if(dup2(nextout, 1) < 0) {
                        perror("dup2");
                    }
                    close(nextout);
                }
                
                /// リダイレクト ///
                for(int i=0; i<vector_size(redirections); i++) {
                    sRedirect* redirect = (sRedirect*)vector_item(redirections, i);
                    
                    mode_t mode;
                    switch(redirect->mType) {
                        case kRedirectInput:
                            mode = O_RDONLY;
                            break;
                        
                        case kRedirectOverwrite:
                            mode = O_RDWR | O_CREAT | O_TRUNC;
                            break;
                            
                        case kRedirectAppend:
                            mode = O_RDWR | O_CREAT | O_APPEND;
                            break;
                    }
                    
                    int openfd = open(redirect->mFName, mode, 0666);
                    if(openfd < 0) {
                        fprintf(stderr, "open %s is err", redirect->mFName);
                        exit(1);
                    }
                    
                    if(openfd != redirect->mFd) {
                        if(dup2(openfd, redirect->mFd) < 0) {
                            perror("dup2");
                        }
                        close(openfd);
                    }
                }

                /// char** に変換 ///
                char** argv2 = (char**)malloc(sizeof(char*)*(vector_size(argv)+1));

                int i;
                for(i=0; i<vector_size(argv); i++) {
                    string_obj* item = (string_obj*)vector_item(argv, i);
                    argv2[i] = string_c_str(item);
//P((f, "(%s)", string_c_str(item)));
                }
                argv2[i] = NULL;

/*
P((f, "子プロセス exec前 pid %d", getpid()));
P((f, "子プロセス exec前 ssid %d", tcgetsid(0)));
P((f, "子プロセス exec前 pgrp %d\n", tcgetpgrp(0)));
*/


//usleep(500000);

                /// exec ///
                execvp(argv2[0], argv2);
                fprintf(stderr, "exec() error");
                exit(1);
            }

            /// 親プロセスの処理 ///
/*
P((f, "親プロセス pid %d", getpid()));
P((f, "親プロセス ssid %d", tcgetsid(0)));
P((f, "親プロセス pgrp %d\n", tcgetpgrp(0)));
*/
            if(job->mPGroup == -1) {
                job->mPGroup = pid;

                // 子を独自プロセスグループに入れる
//P((f, "setpgid(%d, %d)", pid, job->mPGroup));
                if(setpgid(pid, job->mPGroup) < 0) perror("setpgid1");

                // 子のプロセスグループのプロセスをフォアグランドに出す //
                if(!statment->background) {
                    if(tcsetpgrp(0, job->mPGroup)) perror("tcsetpgrp");
                }
            }
            else {
                // 子を独自のプロセスグループに入れる
//P((f, "setpgid(%d, %d)", pid, job->mPGroup));
                if(setpgid(pid, job->mPGroup) < 0) perror("setpgid2");
            }

            job->AddChildProgram(pid, argv);
            
            if(nextin != 0) close(nextin);
            if(nextout != 1) close(nextout);
            
            nextin = pipefds[0];
        }

        /// フォアグランドジョブの処理 ///
        if(!statment->background) {
            gForgroundJob = job;

            wait_forground_job();
        }
        
        if(h != vector_size(statments)-1) {
            if(mis_curses() > 0) {
                printf("\n");
            }
            else {
                printf("\n");
            }
        }
    }

    /// hitanykey ///
    if(hitanykey) {
        if(mis_curses() > 0) {
            printf("\nHIT ANY KEY");
            fflush(stdout);

            int meta;
            mgetch(&meta);
        }
        else {
            printf("\nHIT ANY KEY");
            fflush(stdout);

            minitscr();
            int meta;
            mgetch(&meta);
            mendwin();
        }
    }
    
    /// 端末のタイトルに名前を付ける ///
    if(gChangeTerminalTitle) {
        char tmp2[PATH_MAX];
        char* home = getenv("HOME");
        if(home == NULL) {
            fprintf(stderr, "HOME is NULL");
            exit(1);
        }
        if(strstr(ActiveDir()->Path(), home) == ActiveDir()->Path()) {
            strcpy(tmp2, "~");
            sprintf(tmp2 + strlen(tmp2), ActiveDir()->Path() + strlen(home));
            printf("%c]2;mfiler2 %s%c", 033, tmp2, 007);
            fflush(stdout);
        }
        else {
            printf("%c]2;mfiler2 %s%c", 033, ActiveDir()->Path(), 007);
            fflush(stdout);
        }
    }

    /// 廃棄処理 ///
    for(int i=0; i<vector_size(statments); i++) {
        delete (sStatment*)vector_item(statments, i);
    }
    vector_delete(statments);
}

///////////////////////////////////////////////////////////////////////////////
// 描写
///////////////////////////////////////////////////////////////////////////////
void shell_view()
{
    const int maxy = mgetmaxy();
    const int maxx = mgetmaxx();

    const int size = vector_size(gJobs);
    for(int i=0; i<size; i++) {
        sJob* job = (sJob*)vector_item(gJobs, i);

        const int x = (maxx/(size<5?5:size))*i;
        char buf[1024];
        sprintf(buf, "[%d]%s", i+1, string_c_str(job->mName));
        if(strlen(buf) > maxx/ (size<5?5:size)) {
            buf[maxx/(size<5?5:size)] = 0;
        }
        mmvprintw(maxy-3-gViewHookLineNum, x, "%s", buf);
    }
}

///////////////////////////////////////////////////////////////////////////////
// ジョブを前面に出す
///////////////////////////////////////////////////////////////////////////////
void shell_forground_job(int num)
{
    if(num>=0 && num < vector_size(gJobs)) {
        mendwin();

        termios tty;
        tcgetattr(STDIN_FILENO, &tty);

        mclear_immediately();
        mmove_immediately(0, 0);
        fflush(stdout);

        sJob* job = (sJob*)vector_item(gJobs, num);
        
        if(tcsetpgrp(0, job->mPGroup)) {
            perror("tcsetpgrp");
        }
        
        gForgroundJob = job;
        
        sChildProgram* prog = (sChildProgram*)vector_item(job->mChildPrograms, 0);
        
        kill(prog->mPid, SIGCONT);

        /// 端末のタイトルに名前を付ける ///
        if(gChangeTerminalTitle) {
            printf("%c]2;mfiler2 %s%c", 033, string_c_str(job->mName), 007);
            fflush(stdout);
        }
        
        wait_forground_job();

        /// 端末のタイトルに名前を付ける ///
        if(gChangeTerminalTitle) {
            char tmp2[PATH_MAX];
            char* home = getenv("HOME");
            if(home == NULL) {
                fprintf(stderr, "HOME is NULL");
                exit(1);
            }
            if(strstr(ActiveDir()->Path(), home) == ActiveDir()->Path()) {
                strcpy(tmp2, "~");
                sprintf(tmp2 + strlen(tmp2), ActiveDir()->Path() + strlen(home));
                printf("%c]2;mfiler2 %s%c", 033, tmp2, 007);
                fflush(stdout);
            }
            else {
                printf("%c]2;mfiler2 %s%c", 033, ActiveDir()->Path(), 007);
                fflush(stdout);
            }
        }
        
        tcsetattr(STDIN_FILENO, TCSANOW, &tty);

        minitscr();
        mclear();

        mclear_immediately();
    }
}


///////////////////////////////////////////////////////////////////////////////
// 全てのジョブをkillする
///////////////////////////////////////////////////////////////////////////////
void shell_kill_all_jobs()
{
    for(int i=0; i<vector_size(gJobs); i++) {
        sJob* job = (sJob*)vector_item(gJobs, i);
        
        for(int j=0; j<vector_size(job->mChildPrograms); j++) {
            sChildProgram* prog = (sChildProgram*)vector_item(job->mChildPrograms, j);
            kill(prog->mPid, SIGKILL);
        }
    }
}

///////////////////////////////////////////////////////////////////////////////
// フォアグランドジョブの終了を待つ
///////////////////////////////////////////////////////////////////////////////
static void wait_forground_job()
{
//P((f, "wait_forground_job"));

    if(vector_size(gForgroundJob->mChildPrograms) > 0) {
        while(gForgroundJob && vector_size(gForgroundJob->mChildPrograms) > 0) {
//P((f, "while"));
            sChildProgram* prog = (sChildProgram*)vector_item(gForgroundJob->mChildPrograms, 0);
            int status;
//P((f,"waitpid start %d", prog->mPid));
            //waitpid(prog->mPid, &status, WNOHANG);
            pid_t pid = waitpid(prog->mPid, &status, WUNTRACED|WNOHANG);
            //waitpid(prog->mPid, &status, WUNTRACED|WCONTINUED);
//P((f,"waitpid end"));
            usleep(100000);
            
            if(pid) {
                /// 終了した場合 ///
                if(WIFEXITED(status) || WIFSIGNALED(status)) {
    //P((f,"wait_forground_job WIFEXITED"));
                    gForgroundJob->mRunningProgs--;
                    delete prog;
                    vector_erase(gForgroundJob->mChildPrograms, 0);

                    if(gForgroundJob->mRunningProgs == 0) {
                        for(int i=0; i<vector_size(gJobs); i++) {
                            if(vector_item(gJobs, i) == gForgroundJob) {
                                vector_erase(gJobs, i);
                                break;
                            }
                        }
                        delete gForgroundJob;
                        gForgroundJob = NULL;
            
                        gRDir->Reread();
                        gLDir->Reread();

                        if(tcsetpgrp(0, getpid())) perror("tcsetpgrp");
                    }
                }
                /// シグナルの配送によりフォワグランドジョブが停止した場合 ///
                else if(WIFSTOPPED(status)) {
    //P((f,"wait_forground_job WIFSTOPPED"));
                    gForgroundJob = NULL;

                    if(tcsetpgrp(0, getpid())) perror("tcsetpgrp");
                }
    /*
                /// その他 ///
                else {
                    fprintf(stderr, "unexpected things happened in wait_forground_job\n");
                    exit(1);
                }
    */
            }
        }
    }
}

///////////////////////////////////////////////////////////////////////////////
// バックグラウンドジョブが終了していれば後処理をする
///////////////////////////////////////////////////////////////////////////////
void shell_wait_backgroud_job()
{
/*
    /// waitpidして状態が変化していればjobを消す ///
    int status;
    pid_t pid = waitpid(-1, &status, WNOHANG);

    if(pid > 0) {
        for(int i=0; i<vector_size(gJobs); i++) {
            sJob* job = (sJob*)vector_item(gJobs, i);
            for(int j=0; j<vector_size(job->mChildPrograms); j++) {
                sChildProgram* prg = (sChildProgram*)vector_item(job->mChildPrograms, j);
                if(prg->mPid == pid) {
                    delete prg;
                    vector_erase(job->mChildPrograms, j);

                    job->mRunningProgs--;

                    if(job->mRunningProgs <= 0) {
                        vector_erase(gJobs, i);
                        delete job;
        
                        gRDir->Reread();
                        gLDir->Reread();
                    }

                    goto while_end1;
                }
            }
        }
    }

while_end1:
    int n = 0;
*/
while_start:
    for(int i=0; i<vector_size(gJobs); i++) {
        sJob* job = (sJob*)vector_item(gJobs, i);
        for(int j=0; j<vector_size(job->mChildPrograms); j++) {
            sChildProgram* prg = (sChildProgram*)vector_item(job->mChildPrograms, j);

            int status;
            pid_t pid = waitpid(prg->mPid, &status, WNOHANG);

            if(pid == prg->mPid || pid < 0 && errno == ECHILD) {
                delete prg;
                vector_erase(job->mChildPrograms, j);

                job->mRunningProgs--;

                if(job->mRunningProgs == 0) {
                    vector_erase(gJobs, i);
                    delete job;
    
                    gRDir->Reread();
                    gLDir->Reread();
                }

                goto while_start;
            }
        }
    }
}


